using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{   
    [Authorize(Roles="Administradors, Administratius")]
    public class AdministratiuController : Controller
    {
		private readonly IAdministratiuRepository administratiuRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public AdministratiuController() : this(new AdministratiuRepository())
        {
        }

        public AdministratiuController(IAdministratiuRepository administratiuRepository)
        {
			this.administratiuRepository = administratiuRepository;
        }

        //
        // GET: /Administratiu/

        public ViewResult Index()
        {
            return View(administratiuRepository.All);
        }

        //
        // GET: /Administratiu/Details/5

        public ViewResult Details(int id)
        {
            return View(administratiuRepository.Find(id));
        }

        //
        // GET: /Administratiu/Create

        public ActionResult Create()
        {   
            return View();
        } 

        //
        // POST: /Administratiu/Create

        [HttpPost]
        public ActionResult Create(Administratiu administratiu)
        {
            string error = string.Empty;
            if (ModelState.IsValid) {
                EscolaMVC.Services.GestioUsuaris gusu = new Services.GestioUsuaris();
                if (!gusu.CrearUsuari(administratiu, out error))
                {
                    ViewBag.Error = error;
                    return View();
                }
                return RedirectToAction("Index");
            } 
            else {
                ViewBag.Error = error;
				return View();
			}
        }
        
        //
        // GET: /Administratiu/Edit/5
 
        public ActionResult Edit(int id)
        {
             return View(administratiuRepository.Find(id));
        }

        //
        // POST: /Administratiu/Edit/5

        [HttpPost]
        public ActionResult Edit(Administratiu administratiu)
        {
            if (ModelState.IsValid) {
                administratiuRepository.InsertOrUpdate(administratiu);
                if (!administratiuRepository.Save())
                {   
                    ViewBag.Error = administratiuRepository.Error;
                    return View();
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Administratiu/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(administratiuRepository.Find(id));
        }

        //
        // POST: /Administratiu/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            string UserName = administratiuRepository.Find(id).UserProfile.UserName;
            administratiuRepository.Delete(id);
            administratiuRepository.Save();
            EscolaMVC.Services.GestioUsuaris gusu = new Services.GestioUsuaris();
            string error = string.Empty;
            if (!gusu.EsborrarUsuari(UserName, out error)) {
                ViewBag.Error = error;
                var admin = administratiuRepository.Find(id);
                if (admin != null)
                {
                    return View(admin);
                }
                else {
                    return View(new Administratiu());
                }
            }
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                administratiuRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

