using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{   
    [Authorize(Roles="Administradors, Administratius")]
    public class AlumneController : Controller
    {
		private readonly IAlumneRepository alumneRepository;
        private readonly IPareRepository pareRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public AlumneController() : this(new AlumneRepository(), new PareRepository())
        {
        }

        public AlumneController(IAlumneRepository alumneRepository, IPareRepository pareRepository)
        {
			this.alumneRepository = alumneRepository;
            this.pareRepository = pareRepository;
        }

        //
        // GET: /Alumne/

        public ViewResult Index()
        {
            return View(alumneRepository.AllIncluding(alumne => alumne.Cita, alumne => alumne.Matricula, alumne => alumne.Pare));
        }

        //
        // GET: /Alumne/Details/5

        public ViewResult Details(int id)
        {
            return View(alumneRepository.Find(id));
        }

        public ActionResult Pares(int id)
        {
            Alumne alumne = alumneRepository.Find(id);
            if (alumne != null)
            {
                return View(alumneRepository.Find(id));
            }
            else {
                TempData["Error"] = "No existeix cap alumne amb aquest id";
                return RedirectToAction("Index");
            }
        }

        public ActionResult AfegirPare(int id, int idpare) {
            Alumne a = alumneRepository.Find(id);
            if (a != null) {
                if (!alumneRepository.AfegirPare(a, idpare) || !alumneRepository.Save()) {
                    TempData["Error"] = alumneRepository.Error;   
                }
            }
            return RedirectToAction("Pares", new { id = id});
        }


        [HttpPost]
        public ActionResult Pares(int idalumne, string DNI, string nompare, string cgpare )
        {
            Alumne alumne = alumneRepository.Find(idalumne);
            if (alumne != null)
            {
                IEnumerable<Pare> pares;

                if (DNI != null && DNI != string.Empty)
                {
                    pares = pareRepository.All.Where(p => p.DNI.Contains(DNI)).AsEnumerable();
                }
                else
                {
                    pares = pareRepository.All.Where(p => p.Nom.Contains(nompare) && p.Cognoms.Contains(cgpare)).AsEnumerable();
                }

                ViewBag.Pares = pares;
                return View(alumne);
            }
            else
            {
                TempData["Error"] = "No existeix cap alumne amb aquest id";
                return RedirectToAction("Index");
            }
            

            
        }

        public ActionResult EliminarPare(int id, int idpare)
        {
            Alumne alumne = alumneRepository.Find(id);
            if (alumne != null)
            {
                if (!alumneRepository.EliminarPare(alumne, idpare) || !alumneRepository.Save())
                {
                    TempData["Error"] = alumneRepository.Error;
                    return RedirectToAction("Index");
                }
                return RedirectToAction("Pares", new { id = id });
            }
            else {
                TempData["Error"] = "No existeix cap alumne amb aquest id";
                return RedirectToAction("Index");
            }
            
        }

        //
        // GET: /Alumne/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Alumne/Create

        [HttpPost]
        public ActionResult Create(Alumne alumne)
        {
            if (ModelState.IsValid) {
                alumneRepository.InsertOrUpdate(alumne);
                alumneRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Alumne/Edit/5
 
        public ActionResult Edit(int id)
        {
             return View(alumneRepository.Find(id));
        }

        //
        // POST: /Alumne/Edit/5

        [HttpPost]
        public ActionResult Edit(Alumne alumne)
        {
            if (ModelState.IsValid) {
                alumneRepository.InsertOrUpdate(alumne);
                alumneRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Alumne/Delete/5
 
        public ActionResult Delete(int id)
        {   
            return View(alumneRepository.Find(id));
        }

        //
        // POST: /Alumne/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            alumneRepository.Delete(id);
            alumneRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                alumneRepository.Dispose();
                pareRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

