using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class AssignaturaController : Controller
    {
		private readonly IAssignaturaRepository assignaturaRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public AssignaturaController() : this(new AssignaturaRepository())
        {
        }

        public AssignaturaController(IAssignaturaRepository assignaturaRepository)
        {
			this.assignaturaRepository = assignaturaRepository;
        }

        //
        // GET: /Assignatura/

        public ViewResult Index()
        {
            return View(assignaturaRepository.AllIncluding(assignatura => assignatura.Avaluacio, assignatura => assignatura.Docencia, assignatura => assignatura.Exercici));
        }

        //
        // GET: /Assignatura/Details/5

        public ViewResult Details(int id)
        {
            return View(assignaturaRepository.Find(id));
        }

        //
        // GET: /Assignatura/Create

        public ActionResult Create()
        {
            omplirCursos();
            return View();
        } 

        //
        // POST: /Assignatura/Create

        [HttpPost]
        public ActionResult Create(Assignatura assignatura, int CursId)
        {
            using (CursRepository cursos = new CursRepository())
            {
                Curs curs = cursos.Find(CursId);
                if (curs != null){
                    assignatura.Curs = curs;
                }
            }
            if (ModelState.IsValid) {
                if (!assignaturaRepository.Insert(assignatura) || !assignaturaRepository.Save()) {
                    TempData["Error"] = assignaturaRepository.Error;
                    return RedirectToAction("Create");
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Assignatura/Edit/5
 
        public ActionResult Edit(int id)
        {
            Assignatura assignatura = assignaturaRepository.Find(id);
            if (assignatura == null) {
                TempData["Error"] = "No existeix cap assignatura amb aquest id";
                return RedirectToAction("Index");
            }
            omplirCursos(); 
            return View(assignatura);
        }

        //
        // POST: /Assignatura/Edit/5

        [HttpPost]
        public ActionResult Edit(Assignatura assignatura, int oldCursId, int CursId)
        {
            if (ModelState.IsValid) {
                if (!assignaturaRepository.Update(assignatura, oldCursId, CursId) || !assignaturaRepository.Save())
                {
                    TempData["Error"] = assignaturaRepository.Error;
                    return RedirectToAction("Edit", assignatura.Id);
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Assignatura/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(assignaturaRepository.Find(id));
        }

        //
        // POST: /Assignatura/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            assignaturaRepository.Delete(id);
            assignaturaRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                assignaturaRepository.Dispose();
            }
            base.Dispose(disposing);
        }

        private void omplirCursos()
        {
            CursRepository cursosRep = new CursRepository();
            var elsCursos = cursosRep.All.OrderBy(m => m.Tipus).ThenBy(m => m.Numero).AsEnumerable();
            ViewBag.Cursos = elsCursos;
        }
    }
}

