using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius, Professors")]
    public class AvaluacioController : Controller
    {
		private readonly IAvaluacioRepository avaluacioRepository;
        private readonly ICursRepository cursRepository;
        private readonly IClasseRepository classeRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public AvaluacioController() : this(new AvaluacioRepository(), new CursRepository(), new ClasseRepository())
        {
        }

        public AvaluacioController(IAvaluacioRepository avaluacioRepository, ICursRepository cursRepository, IClasseRepository classeRepository)
        {
			this.avaluacioRepository = avaluacioRepository;
            this.cursRepository = cursRepository;
            this.classeRepository = classeRepository;
        }

        //
        // GET: /Avaluacio/

        [HttpGet]
        public ViewResult Index(int? Any)
        {
            OmplirFiltres(Any);
            return View(avaluacioRepository.All.AsEnumerable());
        }

        /*Gestiona el POST des dels botns Filtrar o Netejar*/
        [HttpPost]
        public ActionResult Index(int? Any, int? CursId, int? ClasseId, string pendents) {
            if (CheckResetButtonPressed) return RedirectToAction("Index");
            var avaluacions = FiltrarResultats(Any,CursId, ClasseId, pendents);
            return View(avaluacions);
        }

        public ActionResult Generar(int Any) {
            TempData["Error"] = null;
            TempData["Generar"] = null;
            try
            {
                int generats = 0;
                avaluacioRepository.Generar(Any,out generats);
                avaluacioRepository.Save();
                TempData["Generar"] = "Avaluacions generades: "+generats.ToString();
            }
            catch(Exception ex) {
                TempData["Error"] = ex.Message;
            }
            
            return RedirectToAction("Index", new {Any = Any});
        
        }


        [HttpPost]
        public ActionResult Guardar(int? FiltreAny, int? FiltreCursId, int? FiltreClasseId, string FiltrePendents, FormCollection notesform)
        {
            string[] keys = notesform.AllKeys.Where(key=>key.StartsWith("Nota")).ToArray();
            Dictionary<string, Avaluacio> avaluacions = new Dictionary<string, Avaluacio>();
            string error ="";
        #region foreach carregar diccionari
            try
            {
                foreach (string key in keys)
                {
                    string[] splitkey = key.Split('_');
                    string idavaluacio = splitkey[2];
                    string tipusnota = splitkey[1];
                    string nota = notesform[key];
                    decimal? notadecimal;
                    #region checks nota correcta o null
                    if (nota == null || nota == "")
                    {
                        notadecimal = -1;
                    }
                    else
                    {
                        notadecimal = decimal.Parse(nota);
                        if (notadecimal < 0 || notadecimal > 10)
                        {
                            error = "Notes incorrectes a " + idavaluacio + "nota: " + nota;
                            break;
                        }
                    }
                    #endregion

                    if (!avaluacions.ContainsKey(idavaluacio))
                    {
                        avaluacions.Add(idavaluacio, new Avaluacio());
                        avaluacions[idavaluacio].Id = int.Parse(idavaluacio);
                    }
                    #region switch tipus de nota
                    /*[ Possibles valors permesos {PrTr, SgTr, TrTr, Fn }]*/
                    switch (tipusnota)
                    {
                        case "PrTr":
                            {
                                avaluacions[idavaluacio].PrTrimestre = notadecimal;
                                break;
                            }
                        case "SgTr":
                            {
                                avaluacions[idavaluacio].SgTrimestre = notadecimal;
                                break;

                            }
                        case "TrTr":
                            {
                                avaluacions[idavaluacio].TrTrimestre = notadecimal;
                                break;
                            }
                        case "Fn":
                            {
                                avaluacions[idavaluacio].Final = notadecimal;
                                break;
                            }
                        default: throw new Exception("Alguna cosa ha anat malament");
                    }
                    #endregion

                }
            }
            catch(Exception ex) {
                error = ex.Message;
            }

    #endregion
            if (error==string.Empty) {
                try
                {
                    avaluacioRepository.CarregarNotes(avaluacions.Values.AsEnumerable());
                    avaluacioRepository.Save();
                }
                catch(Exception ex) {
                    error = ex.Message;
                }
            }

            return RedirectToAction("Index", new { FiltreAny, FiltreCursId, FiltreClasseId, FiltrePendents});
        }

        //
        // GET: /Avaluacio/Details/5

        public ViewResult Details(int id)
        {
            ViewBag.ProfessorMode = true;
            return View(avaluacioRepository.Find(id));
        }
        
        //
        // GET: /Avaluacio/Edit/5
 
        public ActionResult Edit(int id)
        {
            Avaluacio avaluacio = avaluacioRepository.Find(id);
            AvaluacioEditVM avalVm = new AvaluacioEditVM { 
                Id = avaluacio.Id,
                PrTrimestre = avaluacio.PrTrimestre,
                SgTrimestre = avaluacio.SgTrimestre,
                TrTrimestre = avaluacio.TrTrimestre,
                Final = avaluacio.Final,
                Alumne = avaluacio.Matricula.Alumne.ToString(),
                Assignatura = avaluacio.Assignatura.Nom,
                Curs = avaluacio.Assignatura.Curs.ToString(),
                Classe = avaluacio.Matricula.Classe.ToString()
            };
            
            return View(avalVm);
        }

        //
        // POST: /Avaluacio/Edit/5

        [HttpPost]
        public ActionResult Edit(Avaluacio avaluacio)
        {
            if (ModelState.IsValid) {
                avaluacioRepository.InsertOrUpdate(avaluacio);
                avaluacioRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Avaluacio/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(avaluacioRepository.Find(id));
        }

        //
        // POST: /Avaluacio/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            avaluacioRepository.Delete(id);
            avaluacioRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                avaluacioRepository.Dispose();
                cursRepository.Dispose();
                classeRepository.Dispose();
            }
            base.Dispose(disposing);
        }

        private void OmplirFiltres(int?any) {
            int inici = DateTime.Today.Year - 2;
            int fi = inici + 3;
            if (any.HasValue)
            {
                inici = any.Value;
                fi = inici;
            }
            List<int> anys = new List<int>();
            for (int _any = inici; _any <= fi;_any++){anys.Add(_any);}
            
            ViewBag.Anys = anys.ToArray();
            ViewBag.Cursos = cursRepository.All.Where(curs=>curs.AnyCurs >= inici && curs.AnyCurs<=fi).OrderBy(curs=>curs.Tipus).ThenBy(curs=>curs.AnyCurs).ThenBy(curs=>curs.Numero).AsEnumerable();
            ViewBag.Classes = classeRepository.All.Where(classe => classe.Curs.AnyCurs >= inici && classe.Curs.AnyCurs <= fi).OrderBy(classe=>classe.Curs.AnyCurs).ThenBy(classe=>classe.Curs.Tipus).ThenBy(classe=>classe.Curs.Numero).ThenBy(classe=>classe.Lletra).AsEnumerable();
        }
        private void FiltresToViewBag(int? Any, int? CursId, int? ClasseId, string pendents)
        {
            ViewBag.Any = Any;
            ViewBag.CursId = CursId.HasValue ? CursId.Value : 0;
            ViewBag.ClasseId = ClasseId.HasValue ? ClasseId.Value : 0;
            ViewBag.Pendents = pendents;
        }

        private IEnumerable<Avaluacio> FiltrarResultats(int? Any, int? CursId, int? ClasseId, string pendents)
        {               
            var avaluacions = avaluacioRepository.AllIncluding(avaluacio => avaluacio.ComentariAvaluacio);
            avaluacions = avaluacions.Where(avaluacio => avaluacio.Matricula.Any == Any);
            if (CursId.HasValue)
            {
                avaluacions = avaluacions.Where(avaluacio => avaluacio.Assignatura.Curs.Id == CursId);
            }
            if (ClasseId.HasValue)
            {
                avaluacions = avaluacions.Where(avaluacio => avaluacio.Matricula.Classe.Id == ClasseId);
            }
            FiltresToViewBag(Any, CursId, ClasseId, pendents);
            OmplirFiltres(Any);
            return avaluacions.AsEnumerable();
        }

        private bool CheckResetButtonPressed{
            get{
                return (this.HttpContext.Request["Neteja"] != null);
            }
        
        }
        
        
            

        
        
    }
}

