using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class ClasseController : Controller
    {
		private readonly IClasseRepository classeRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public ClasseController() : this(new ClasseRepository())
        {
        }

        public ClasseController(IClasseRepository classeRepository)
        {
			this.classeRepository = classeRepository;
        }

        //
        // GET: /Classe/

        public ViewResult Index()
        {
            return View(classeRepository.AllIncluding(classe => classe.Curs));
        }

        //
        // GET: /Classe/Details/5

        public ViewResult Details(int id)
        {
            return View(classeRepository.Find(id));
        }

        //
        // GET: /Classe/Create

        public ActionResult Create()
        {
            omplirCursos();
            return View();
        } 

        //
        // POST: /Classe/Create

        [HttpPost]
        public ActionResult Create(Classe classe, int CursId)
        {   
            using (CursRepository cursos = new CursRepository())
            {   
                Curs curs = cursos.Find(CursId);
                if (curs != null){
                    classe.Curs = curs;
                }
            }
            if (ModelState.IsValid) {
                if (!classeRepository.Insert(classe) || !classeRepository.Save()){   
                    TempData["Error"] = classeRepository.Error;
                    return RedirectToAction("Create");
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Classe/Edit/5
 
        public ActionResult Edit(int id)
        {   
            Classe classe = classeRepository.Find(id);
            if (classe == null)
            {
                TempData["Error"] = "No existeix cap classe amb aquest id";
                return RedirectToAction("Index");
            }
            omplirCursos();
            return View(classe);
        }

        //
        // POST: /Classe/Edit/5

        [HttpPost]
        public ActionResult Edit(Classe classe,int oldCursId, int CursId)
        {   
            if (ModelState.IsValid) {
                if (!classeRepository.Update(classe, oldCursId, CursId) || !classeRepository.Save()) {
                    TempData["Error"] = classeRepository.Error;
                    return RedirectToAction("Edit", classe.Id);
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Classe/Tutor/5

        public ActionResult Tutor(int id)
        {
            Classe classe = classeRepository.Find(id);
            if (classe == null)
            {
                TempData["Error"] = "No existeix cap classe amb aquest id";
                return RedirectToAction("Index");
            }
            omplirProfessors();
            return View(classe);
        }
        
        [HttpPost]
        public ActionResult Tutor(int ClasseId, int Professorid)
        {
            Classe classe = classeRepository.Find(ClasseId);
            if (classe == null)
            {
                TempData["Error"] = "No existeix cap classe amb aquest id";
            }
            else{
                if (!classeRepository.AssignarTutor(classe, Professorid) || !classeRepository.Save()) {
                    TempData["Error"] = classeRepository.Error;
                }
            }
            return RedirectToAction("Index");
        }

        public ActionResult EliminarTutor(int id) {

            Classe classe = classeRepository.Find(id);
            if (classe == null) {
                TempData["Error"] = "No hi ha cap classe amb aquest id.";
            }
            if (!classeRepository.EliminarTutor(classe) || !classeRepository.Save()) {
                TempData["Error"] = classeRepository.Error;
            }
            return RedirectToAction("Index");
        }



        //
        // GET: /Classe/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(classeRepository.Find(id));
        }

        //
        // POST: /Classe/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            classeRepository.Delete(id);
            classeRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                classeRepository.Dispose();
            }
            base.Dispose(disposing);
        }

        private void omplirCursos() {
            CursRepository cursosRep = new CursRepository();
            var elsCursos = cursosRep.All.OrderBy(m => m.Tipus).ThenBy(m => m.Numero).AsEnumerable();
            ViewBag.Cursos = elsCursos;
            
        }
        private void omplirProfessors() {
            ProfessorRepository professorRep = new ProfessorRepository();
            var elsProfes = professorRep.All.OrderBy(p => p.Cognoms).ThenBy(p => p.Nom).AsEnumerable();
            ViewBag.Professors = elsProfes;
        }
    }


}

