using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class CursController : Controller
    {
		private readonly ICursRepository cursRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public CursController() : this(new CursRepository())
        {
        }

        public CursController(ICursRepository cursRepository)
        {
			this.cursRepository = cursRepository;
        }

        //
        // GET: /Curs/

        public ViewResult Index()
        {
            return View(cursRepository.AllIncluding(curs => curs.Assignatura, curs => curs.Classe));
        }

        //
        // GET: /Curs/Details/5

        public ViewResult Details(int id)
        {
            return View(cursRepository.Find(id));
        }

        //
        // GET: /Curs/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Curs/Create

        [HttpPost]
        public ActionResult Create(Curs curs)
        {
            if (ModelState.IsValid) {
                cursRepository.InsertOrUpdate(curs);
                cursRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Curs/Edit/5
 
        public ActionResult Edit(int id)
        {
             return View(cursRepository.Find(id));
        }

        //
        // POST: /Curs/Edit/5

        [HttpPost]
        public ActionResult Edit(Curs curs)
        {
            if (ModelState.IsValid) {
                cursRepository.InsertOrUpdate(curs);
                cursRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Curs/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(cursRepository.Find(id));
        }

        //
        // POST: /Curs/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            cursRepository.Delete(id);
            cursRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                cursRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

