using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class DocenciaController : Controller
    {
		private readonly IDocenciaRepository docenciaRepository;
        private readonly IProfessorRepository professorRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public DocenciaController() : this(new DocenciaRepository(), new ProfessorRepository())
        {
        }

        public DocenciaController(IDocenciaRepository docenciaRepository, IProfessorRepository professorRepository)
        {
			this.docenciaRepository = docenciaRepository;
            this.professorRepository = professorRepository;
        }

        //
        // GET: /Docencia/

        public ViewResult Index()
        {
            omplirProfessors();
            return View(docenciaRepository.All);
        }

        [HttpPost]
        public ViewResult Index(string assignatura, int? curs,string tipus, string classe, int? any, string lliures)
        {
            omplirProfessors();
            IEnumerable<Docencia> docencies = docenciaRepository.All.AsEnumerable();
            if (assignatura != "") {
                docencies = docencies.Where(docencia => docencia.Assignatura.Nom == assignatura);
            }
            if (curs!=null) {
                docencies = docencies.Where(docencia => docencia.Assignatura.Curs.Numero == curs);
            }
            if (tipus != "") {
                docencies = docencies.Where(docencia => docencia.Assignatura.Curs.Tipus == tipus);
            }
            if (classe != "")
            {
                docencies = docencies.Where(docencia => docencia.Classe.Lletra == classe);
            }
            if (any != null) {
                docencies = docencies.Where(docencia => docencia.Assignatura.Curs.AnyCurs == any);
            }
            if (lliures != null) {
                docencies = docencies.Where(docencia => docencia.Professor == null);
            }
            return View(docencies);
            
        }

        public ActionResult Generar() {
            docenciaRepository.Generar(2013);
            if (!docenciaRepository.Save()) {
                TempData["Error"] = docenciaRepository.Error;
            }
            return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult Guardar(FormCollection plantilla) {
            foreach (var key in plantilla.AllKeys) {
                string strprofe = plantilla[key];
                if (strprofe != "") {
                    int ProfessorId = int.Parse(strprofe);
                    int DocenciaId = int.Parse(key);
                    if (!docenciaRepository.AfegirProfessor(DocenciaId, ProfessorId)) {
                        TempData["Error"] = docenciaRepository.Error;
                        return RedirectToAction("Index");
                    }
                }
            }
            if (!docenciaRepository.Save()) {
                TempData["Error"] = docenciaRepository.Error; 
            }
            
            return RedirectToAction("Index");
        }

        //
        // GET: /Docencia/Details/5

        public ViewResult Details(int id)
        {
            return View(docenciaRepository.Find(id));
        }

        //
        // GET: /Docencia/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Docencia/Create

        [HttpPost]
        public ActionResult Create(Docencia docencia)
        {
            if (ModelState.IsValid) {
                docenciaRepository.InsertOrUpdate(docencia);
                docenciaRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Docencia/Edit/5
 
        public ActionResult Edit(int id)
        {
            omplirProfessors(); 
            return View(docenciaRepository.Find(id));
        }

        //
        // POST: /Docencia/Edit/5

        [HttpPost]
        public ActionResult Edit(Docencia docencia, int ProfessorId)
        {
            if (ModelState.IsValid) {
                docenciaRepository.AfegirProfessor(docencia.Id, ProfessorId);
                docenciaRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Docencia/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(docenciaRepository.Find(id));
        }

        //
        // POST: /Docencia/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            docenciaRepository.Delete(id);
            docenciaRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                docenciaRepository.Dispose();
                professorRepository.Dispose();
            }
            base.Dispose(disposing);
        }
        private void omplirProfessors() {
            ViewBag.Professors = professorRepository.All.ToList();
        }
    }
}

