﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http;
using EscolaMVC.Entitats;
using EscolaMVC.Models;
using EscolaMVC.Services;

namespace EscolaMVC.Controllers
{
    public class CitaDTO
    {
        public int id;
        public string dia;
        public string professor;
        public string fill;
        public string estat;
        public string comentari;
        public string historic;
    }

    public class CitaProfessorDTO {
        public int id;
        public string dia;
        public string[] pares;
        public string alumne;
        public string estat;
        public string comentari;
        public string historic;
    }

    public class GestorCitesController : ApiController
    {

        PortalPares portalParesDomini = new PortalPares();
        PortalProfessors portalProfessorDomini = new PortalProfessors();

        
        [Authorize(Roles="Pares")]
        [HttpGet]
        public IEnumerable<CitaDTO>  citesPare(int PareId, int year, int month)
        {
            System.Threading.Thread.Sleep(400);
            IEnumerable<Cita> citas = portalParesDomini.getCitasPare(PareId, year, month).ToList();

            var Citas = from cita in citas
                            select new CitaDTO
                            {
                                id = cita.Id,
                                dia = cita.Dia.HasValue ? cita.Dia.Value.ToString("dd/MM/yyyy") : "",
                                professor = cita.Professor.Nom + " " + cita.Professor.Cognoms,
                                fill = cita.Alumne.Nom + " " + cita.Alumne.Cognoms,
                                estat = cita.NomEstat,
                                comentari = cita.Comentari,
                                historic = cita.Historic == null ? "" : cita.Historic
                            };


            return Citas.AsEnumerable();
        }

        private string[] getPares(Alumne alumne) {
            return alumne.Pare.Select(p => p.Cognoms + ", " + p.Nom).ToArray();
        }
        
        [HttpGet]
        public void acceptarCita(int id)
        {
            string error = "";
            portalParesDomini.acceptarCita(id, out error);
        }
        
        [HttpDelete]
        public HttpResponseMessage eliminarCita(int id)
        {
            string error = "";
            portalParesDomini.eliminarCita(id, out error);
            return new HttpResponseMessage(HttpStatusCode.OK);
        }
        
        [HttpGet]
        public void cancelarCita(int id, string motius)
        {
            string error = "";
            portalParesDomini.cancelarCita(id, motius, out error);
        }

        [HttpGet]
        public void rebutjarCita(int id, string motius)
        {
            string error = "";
            portalParesDomini.rebutjarCita(id, motius, out error);
        }

        
        [Authorize(Roles = "Professors")]
        [HttpGet]
        public IEnumerable<CitaProfessorDTO> citesProfessor(int ProfessorId, int year, int month)
        {
            System.Threading.Thread.Sleep(400);
            IEnumerable<Cita> citas = portalProfessorDomini.getCitasProfessor(ProfessorId, year, month).ToList();

            var Citas = from cita in citas
                        select new CitaProfessorDTO
                        {
                            id = cita.Id,
                            dia = cita.Dia.HasValue ? cita.Dia.Value.ToString("dd/MM/yyyy") : "",
                            pares = getPares(cita.Alumne),
                            alumne = cita.Alumne.Nom + " " + cita.Alumne.Cognoms,
                            estat = cita.NomEstat,
                            comentari = cita.Comentari,
                            historic = cita.Historic == null ? "" : cita.Historic
                        };


            return Citas.AsEnumerable();
        }
        
        [HttpGet]
        public void acceptarCitaProfessor(int id)
        {
            string error = "";
            portalProfessorDomini.acceptarCita(id, out error);
        }
        
        [HttpGet]
        public void rebutjarCitaProfessor(int id, string motius)
        {
            string error = "";
            portalProfessorDomini.rebutjarCita(id, motius, out error);
        }

        [HttpGet]
        public void cancelarCitaProfessor(int id, string motius)
        {
            string error = "";
            portalProfessorDomini.cancelarCita(id, motius, out error);
        }

        [HttpDelete]
        public HttpResponseMessage eliminarCitaProfessor(int id)
        {
            string error = "";
            portalProfessorDomini.eliminarCita(id, out error);
            return new HttpResponseMessage(HttpStatusCode.OK);
        }

        
        
        
        
        protected override void Dispose(bool disposing)
        {
            portalParesDomini.Dispose();
            portalProfessorDomini.Dispose();
            
            base.Dispose(disposing);
        }
    }
}