﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Models;
using EscolaMVC.Entitats;

namespace EscolaMVC.Controllers
{
    public class HomeController : Controller
    {
        private readonly IEscolaSingleTonRepository escolaSingleTonRepository;

        public HomeController() : this(new EscolaSingleTonRepository()) { }

        public HomeController(IEscolaSingleTonRepository escolaSingleTonRepository)
        {
            this.escolaSingleTonRepository = escolaSingleTonRepository;
        }
        
        public ActionResult Index()
        {
            List<Noticia> noticies = new List<Noticia>();
            using (NoticiaRepository noticiesRep = new NoticiaRepository()) {
                DateTime inici = DateTime.Today.AddDays(-30);
                noticies.AddRange(noticiesRep.All.Where(n => n.Data > inici).OrderByDescending(n => n.Data).ToList());
            }
            if (noticies.Any())
                ViewBag.Noticies = noticies;
            return View();
        }

        public ActionResult About()
        {
            Escola escola = null;
            using(EscolaSingleTonRepository escolaRep=new EscolaSingleTonRepository()){
                escola = escolaRep.Escola;
            }
            return View(escola);
        }

        public ActionResult Contact()
        {
            FormulariContacteVM fcVm = new FormulariContacteVM(escolaSingleTonRepository.Escola);
            
            return View(fcVm);
        }
        
        [HttpPost]
        public ActionResult Contact(FormulariContacteVM fcVm)
        {
            string correu_remitent = fcVm.Correu;
            string nom_i_cognoms = fcVm.NomICognoms;
            string assumpte = fcVm.Assumpte;
            string cos = correu_remitent + "\n" + fcVm.Texte;
            string error = string.Empty;
            fcVm = new FormulariContacteVM(escolaSingleTonRepository.Escola);
            fcVm.Correu = correu_remitent;
            fcVm.NomICognoms = nom_i_cognoms;
            fcVm.Assumpte = assumpte;
            fcVm.Texte = cos;
            fcVm.Resposta = "Missatge enviat correctament.";
            fcVm.MostrarForm = false;
            try
            {
                /* Cal revisar el fitxer de configuració i que els parametres siguin valids i correctes. */                
                Services.Correu clientCorreu = new Services.Correu(new Services.WebConfigSmtpClient());
                if (!clientCorreu.EnviarCorreu(correu_remitent, nom_i_cognoms, assumpte, cos, out error))
                {
                    fcVm.Resposta = "";
                    fcVm.Error = error;
                    fcVm.MostrarForm = true;
                }
            }
            catch(Exception ex) {
                fcVm.Resposta = "";
                fcVm.Error = ex.Message;
                fcVm.MostrarForm = true;
            }
            return View(fcVm);
        }
        
    }
}
