using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class MatriculaController : Controller
    {
		private readonly IMatriculaRepository matriculaRepository;
        private readonly IClasseRepository  classesRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public MatriculaController() : this(new MatriculaRepository(), new ClasseRepository())
        {

        }

        public MatriculaController(IMatriculaRepository matriculaRepository, IClasseRepository classeRepository)
        {
			this.matriculaRepository = matriculaRepository;
            this.classesRepository = classeRepository;
        }

        //
        // GET: /Matricula/

        public ViewResult Index()
        {   
            MatriculaVM matriculaViewModel = new MatriculaVM();
            matriculaViewModel.Classes = classesRepository.All.AsEnumerable();
            matriculaViewModel.Any = DateTime.Today.Year;
            IEnumerable<Matricula> matricules = matriculaRepository.AllIncluding(matricula => matricula.Alumne, matricula => matricula.Classe).Where(m=>m.Any==DateTime.Today.Year).AsEnumerable();
            matriculaViewModel.Matricules = matricules;
            return View(matriculaViewModel);
        }

        [HttpPost]
        public ViewResult Index(int Any, int IdAlumne, string Alumne)
        {
            
            IEnumerable<Matricula> matricules;
            if (IdAlumne != 0 && Alumne!=string.Empty)
            {
                matricules = matriculaRepository.AllIncluding(matricula => matricula.Alumne, matricula => matricula.Classe).Where(m => m.Any == Any && m.Alumne.Id == IdAlumne).AsEnumerable();
            }
            else {
                matricules = matriculaRepository.AllIncluding(matricula => matricula.Alumne, matricula => matricula.Classe).Where(m => m.Any == Any).AsEnumerable();
            }
            MatriculaVM matriculaViewModel = new MatriculaVM();
            matriculaViewModel.Matricules = matricules;
            matriculaViewModel.Any = Any;
            matriculaViewModel.IdAlumne = 0;
            matriculaViewModel.Classes = classesRepository.All.AsEnumerable();
            return View(matriculaViewModel);
        }

        [HttpPost]
        public ActionResult Generar(int Any)
        {   
            matriculaRepository.Generar(Any);
            try { 
                matriculaRepository.Save();
                TempData["Success"] = "Correcte";
            }
            catch(Exception ex) {
                TempData["Error"] = ex.Message;
            }
            return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult Guardar(FormCollection plantilla) {
            foreach (var key in plantilla.AllKeys)
            {
                string strClasse = plantilla[key];
                if (strClasse != "")
                {
                    int ClasseId = int.Parse(strClasse);
                    int MatriculaId = int.Parse(key);
                    if (!matriculaRepository.AssignarClasse(MatriculaId, ClasseId))
                    {
                        TempData["Error"] = matriculaRepository.Error;
                        return RedirectToAction("Index");
                    }
                }
            }
            if (matriculaRepository.Save())
            {
                TempData["Success"] = "S'han guardat les dades";
            }
            else {
                TempData["Error"] = matriculaRepository.Error;
            }
            return RedirectToAction("Index"); 
        }


        //
        // GET: /Matricula/Details/5

        public ViewResult Details(int id)
        {
            return View(matriculaRepository.Find(id));
        }

        //
        // GET: /Matricula/Create

        public ActionResult Create()
        {   
            ViewBag.Classes = classesRepository.All.AsEnumerable();
            return View();
        } 

        //
        // POST: /Matricula/Create

        [HttpPost]
        public ActionResult Create(Matricula matricula, int ClasseId)
        {   
            if (ModelState.IsValid) {
                matriculaRepository.Insert(matricula, ClasseId);
                matriculaRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Matricula/Edit/5
 
        public ActionResult Edit(int id)
        {
            ViewBag.Classes = classesRepository.All.AsEnumerable(); 
            return View(matriculaRepository.Find(id));
        }

        //
        // POST: /Matricula/Edit/5

        [HttpPost]
        public ActionResult Edit(int Id, int Any, int? IdClasse)
        {
            int ClasseId = -1;
            if (IdClasse != null) { ClasseId = IdClasse.Value; }
            if (ModelState.IsValid) {
                matriculaRepository.CanviarMatricula(Id, Any, ClasseId);
                matriculaRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Matricula/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(matriculaRepository.Find(id));
        }

        //
        // POST: /Matricula/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            matriculaRepository.Delete(id);
            matriculaRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                matriculaRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

