using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{

    [Authorize(Roles = "Administradors, Administratius")]
    public class NoticiaController : Controller
    {
		private readonly INoticiaRepository noticiaRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public NoticiaController() : this(new NoticiaRepository())
        {
        }

        public NoticiaController(INoticiaRepository noticiaRepository)
        {
			this.noticiaRepository = noticiaRepository;
        }

        //
        // GET: /Noticia/

        public ViewResult Index()
        {
            DateTime inici = DateTime.Today.AddDays(-30);
            return View(noticiaRepository.All.Where(n=>n.Data >= inici));
        }
        
        [HttpPost]
        public ActionResult Index(string inici, string final)
        {
            try
            {
                DateTime ini = DateTime.Parse(inici);
                DateTime fi = DateTime.Parse(final);
                return View(noticiaRepository.All.Where(n => n.Data >= ini && n.Data <= fi));
            }
            catch {
                return RedirectToAction("Index");
            }
            
        }

        //
        // GET: /Noticia/Details/5

        public ViewResult Details(int id)
        {
            return View(noticiaRepository.Find(id));
        }

        //
        // GET: /Noticia/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Noticia/Create

        [HttpPost]
        public ActionResult Create(Noticia noticia)
        {
            if (ModelState.IsValid) {
                noticiaRepository.InsertOrUpdate(noticia);
                noticiaRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }
        
        //
        // GET: /Noticia/Edit/5
 
        public ActionResult Edit(int id)
        {
             return View(noticiaRepository.Find(id));
        }

        //
        // POST: /Noticia/Edit/5

        [HttpPost]
        public ActionResult Edit(Noticia noticia)
        {
            if (ModelState.IsValid) {
                noticiaRepository.InsertOrUpdate(noticia);
                noticiaRepository.Save();
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Noticia/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(noticiaRepository.Find(id));
        }

        //
        // POST: /Noticia/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            noticiaRepository.Delete(id);
            noticiaRepository.Save();

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                noticiaRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

