using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class PareController : Controller
    {
		private readonly IPareRepository pareRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public PareController() : this(new PareRepository())
        {
        }

        public PareController(IPareRepository pareRepository)
        {
			this.pareRepository = pareRepository;
        }

        //
        // GET: /Pare/

        public ViewResult Index()
        {
            return View(pareRepository.AllIncluding(pare => pare.Alumne));
        }

        //
        // GET: /Pare/Details/5

        public ViewResult Details(int id)
        {
            return View(pareRepository.Find(id));
        }

        //
        // GET: /Pare/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Pare/Create

        [HttpPost]
        public ActionResult Create(Pare pare)
        {
            string error = string.Empty;
            if (ModelState.IsValid) {
                EscolaMVC.Services.GestioUsuaris gusu = new Services.GestioUsuaris();
                if (!gusu.CrearUsuari(pare, out error))
                {
                    ViewBag.Error = error;
                    return View();
                }
                return RedirectToAction("Index");
            } 
            else {
                ViewBag.Error = error;
				return View();
			}
        }
        
        //
        // GET: /Pare/Edit/5
 
        public ActionResult Edit(int id)
        {
             return View(pareRepository.Find(id));
        }

        //
        // POST: /Pare/Edit/5

        [HttpPost]
        public ActionResult Edit(Pare pare)
        {
            if (ModelState.IsValid) {
                pareRepository.InsertOrUpdate(pare);
                if (!pareRepository.Save()) {
                    ViewBag.Error = pareRepository.Error;
                    return View();
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Pare/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(pareRepository.Find(id));
        }

        //
        // POST: /Pare/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {   
            string UserName = "";
            Pare p = pareRepository.Find(id);
            if(p.UserProfile!=null){
                UserName = p.UserProfile.UserName;
            }
            
            pareRepository.Delete(id);
            pareRepository.Save();



            if (UserName != "")
            {
                EscolaMVC.Services.GestioUsuaris gusu = new Services.GestioUsuaris();
                string error = string.Empty;
                if (!gusu.EsborrarUsuari(UserName, out error))
                {
                    ViewBag.Error = error;
                    var pare = pareRepository.Find(id);
                    if (pare != null)
                    {
                        return View(pare);
                    }
                    else
                    {
                        return View(new Pare());
                    }
                }
            }
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                pareRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

