using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using EscolaMVC.Entitats;
using EscolaMVC.Models;

namespace EscolaMVC.Controllers
{
    [Authorize(Roles = "Administradors, Administratius")]
    public class ProfessorController : Controller
    {
		private readonly IProfessorRepository professorRepository;

		// If you are using Dependency Injection, you can delete the following constructor
        public ProfessorController() : this(new ProfessorRepository())
        {
        }

        public ProfessorController(IProfessorRepository professorRepository)
        {
			this.professorRepository = professorRepository;
        }

        //
        // GET: /Professor/

        public ViewResult Index()
        {
            return View(professorRepository.AllIncluding(professor => professor.Classe, professor => professor.UserProfile));
        }

        //
        // GET: /Professor/Details/5

        public ViewResult Details(int id)
        {
            return View(professorRepository.Find(id));
        }

        //
        // GET: /Professor/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Professor/Create

        [HttpPost]
        public ActionResult Create(Professor professor)
        {
            string error = string.Empty;
            if (ModelState.IsValid) {
                EscolaMVC.Services.GestioUsuaris gusu = new Services.GestioUsuaris();
                if (!gusu.CrearUsuari(professor, out error))
                {
                    ViewBag.Error = error;
                    return View();
                }
                return RedirectToAction("Index");
            } 
            else {
                ViewBag.Error = error;
				return View();
			}
        }
        
        //
        // GET: /Professor/Edit/5
 
        public ActionResult Edit(int id)
        {
             return View(professorRepository.Find(id));
        }

        //
        // POST: /Professor/Edit/5

        [HttpPost]
        public ActionResult Edit(Professor professor)
        {
            if (ModelState.IsValid) {
                professorRepository.InsertOrUpdate(professor);
                if (!professorRepository.Save()) {
                    ViewBag.Error = professorRepository.Error;
                    return View();
                }
                return RedirectToAction("Index");
            } else {
				return View();
			}
        }

        //
        // GET: /Professor/Delete/5
 
        public ActionResult Delete(int id)
        {
            return View(professorRepository.Find(id));
        }

        //
        // POST: /Professor/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            string UserName = "";
            Professor p = professorRepository.Find(id);

            if(p.UserProfile!=null){
                UserName = p.UserProfile.UserName;
            }
            professorRepository.Delete(id);
            if(!professorRepository.Save()){
                ViewBag.Error = professorRepository.Error;
                var professor = professorRepository.Find(id);
                if (professor != null)
                {
                    return View(professor);
                }
                else
                {
                    return View(new Professor());
                }
            }


            if (UserName != "")
            {
                EscolaMVC.Services.GestioUsuaris gusu = new Services.GestioUsuaris();
                string error = string.Empty;
                if (!gusu.EsborrarUsuari(UserName, out error))
                {
                    ViewBag.Error = error;
                    var professor = professorRepository.Find(id);
                    if (professor != null)
                    {
                        return View(professor);
                    }
                    else
                    {
                        return View(new Professor());
                    }
                }
            }
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                professorRepository.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}

