﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.ComponentModel.DataAnnotations;

namespace EscolaMVC.Entitats
{
    [MetadataType(typeof(EscolaMeta))]
    public partial class Escola {
        
        private class EscolaMeta {
            [Required]
            [Range(0,0)]
            public int Id
            {
                get;
                set;
            }

            [Required(ErrorMessage="El camp nom és obligatori.")]
            [StringLength(50, MinimumLength=1,ErrorMessage="De 1 a 50 caracters.")]
            public string Nom { get; set; }
            
            [Required(ErrorMessage="Cal informar l'adreça.")]
            [StringLength(300, MinimumLength = 1, ErrorMessage = "De 1 a 300 caracters.")]
            public string Adreca { get; set; }

            [Required(ErrorMessage="Cal informar el telèfon.")]
            [StringLength(9, MinimumLength = 1, ErrorMessage = "De 1 a 50 caracters.")]
            public string Telefon { get; set; }
            
            [DataType(DataType.EmailAddress)]
            [StringLength(100, MinimumLength = 1, ErrorMessage = "De 1 a 100 caracters.")]
            [RegularExpression(@"^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$", ErrorMessage = "No és una adreça vàlida.")]
            public string Correu { get; set; }

            [DataType(DataType.Url)]
            public string Url { get; set; }
            
            [AllowHtml]
            [DataType(DataType.MultilineText)]
            public string Info { get; set; }
        }
    
    }
    
    [MetadataType(typeof(AlumneMeta))]
    public partial class Alumne
    {
        public override string ToString()
        {
            return this.Cognoms + ", " + this.Nom;
        }
        
        private class AlumneMeta
        {

            [Required(ErrorMessage="Has d'informar el número d'identificació de l'alumnat. ")]
            public int NIA { get; set; }

            [Required(ErrorMessage="Cal informar el nom de l'alumne.")]
            [StringLength(50, MinimumLength = 1, ErrorMessage = "Màxim 50 caracters.")]
            public string Nom { get; set; }
            [Required(ErrorMessage="Cal informar els cognoms.")]
            [StringLength(150, MinimumLength = 1, ErrorMessage = "Màxim 150 caracters.")]
            public string Cognoms { get; set; }

            [Required(ErrorMessage="Has d'indicar el sexe.")]
            [RegularExpression(@"[H]|[D]", ErrorMessage = "Sexe = H o D.")]
            public string Sexe { get; set; }

            [DataType(DataType.Date), DisplayFormat( DataFormatString="{0:dd/MM/yyyy}", ApplyFormatInEditMode=true )]
            public DateTime Data_naix
            {
                get;
                set;
            }

        }
    }

    class UsuariMeta
    {
        [Required(ErrorMessage="Es obligatori informar el DNI.")]
        [StringLength(9, MinimumLength = 9, ErrorMessage = "El DNI té 9 caràcters.")]
        public string DNI { get ;  set; }
        [Required(ErrorMessage="Has d'informar el nom d'usuari.")]
        [StringLength(50, MinimumLength = 1, ErrorMessage = "Màxim 50 caracters.")]
        public string Nom { get; set; }
        [Required(ErrorMessage="Has d'informar els cognoms.")]
        [StringLength(150, MinimumLength = 1, ErrorMessage = "Màxim 150 caracters.")]
        public string Cognoms { get; set; }
        
        [Required(ErrorMessage="Has d'indicar el sexe.")]
        [RegularExpression(@"[H]|[D]",ErrorMessage="Sexe = H o D.")]
        public string Sexe { get; set; }
        [Required]
        [RegularExpression(@"^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,4})$",ErrorMessage="No és una adreça vàlida." )]
        public string Correu { get; set; }
    }
    
    [MetadataType(typeof(UsuariMeta))]
    public partial class Pare { } 

    [MetadataType(typeof(UsuariMeta))]
    public partial class Administratiu { }

    [MetadataType(typeof(UsuariMeta))]
    public partial class Professor {
        public override string ToString()
        {
            return this.Cognoms + ", "+ this.Nom + "(" + this.Id + ")" ;
        }
    
    }

    public partial class UsuariAp {
        public override string ToString() {
            return this.UserName;
        }
    
    }
    
    
    public class TipusUsuaris {
        private static string[] _tipus = { "Administratiu","Professor","Pare"};
        public static string[] Tipus {
            get { 
                return _tipus; 
            }
        } 
    
    }

    public class RolsUsuari {
        private static string[] _rols = { "Administradors","Administratius", "Professors", "Pares", "Alumnes" };
        private static string[] _rolsgestio = { "Administradors", "Administratius", "Professors"};

        public static string RolAdministrador { get { return "Administradors"; } }
        public static string RolAdministratiu { get { return "Administratius"; } }
        public static string RolProfessor { get { return "Professors"; } }
        public static string RolPare { get { return "Pares"; } }
        public static string RolAlumne { get { return "Alumnes"; } }

        public static string[] Rols
        {
            get
            {
                return _rols;
            }
        }
        public static string[] RolsGestio {
            get
            {
                return _rolsgestio;
            }
        }

    }

    [MetadataType(typeof(CursMeta))]
    public partial class Curs {


        static string[] _tipus = { "Primària" , "Secundària", "ESO", "Batxillerat" };
        public static string[] TipusCurs{
            get{
                return _tipus;
            }
        }

        public override string ToString()
        {
            return this.Numero + " " + this.Tipus + ", " + this.AnyCurs;
            
        }

        private class CursMeta {
            [Display(Name="Any")]
            public int AnyCurs{get;set;}
        
        }

        public int Matriculats {
            get {
                return this.Classe.Select(c => c.Matricula.Count()).Sum();
            }
        }
    }

    [MetadataType(typeof(ClasseMeta))]
    public partial class Classe {
        private class ClasseMeta {
            [Required]
            [StringLength(1,MinimumLength=1, ErrorMessage="El camp lletra ha de ser una lletra...")]
            [RegularExpression(@"[A-Z]", ErrorMessage = "Rang vàlid A-Z.")]
            public string Lletra
            {
                get;
                set;
            }

        }

        public override string ToString()
        {
            if (this.Curs != null)
            {
                return this.Curs.Numero + " - "+ this.Lletra + " " + this.Curs.Tipus + " " + this.Curs.AnyCurs;
            }
            else return this.Lletra;
        }
    }

    [MetadataType(typeof(AssignaturaMeta))]
    public partial class Assignatura {
        public override string ToString()
        {
            if (this.Curs != null)
            {
                return this.Nom + " " + this.Curs.Numero + " " + this.Curs.Tipus;
            }
            else {
                return this.Nom;
            }
        }
        private class AssignaturaMeta {
            [Required(ErrorMessage="Camp {0} requerit.")]
            [StringLength(100, MinimumLength = 1, ErrorMessage = "Màxim {1} caracters.")]
            public string Nom { get; set; }
        
        }
    
    }

    [MetadataType(typeof(NoticiaMeta))]
    public partial class Noticia{
        private class NoticiaMeta {

            [DataType(DataType.Date)]
            [Required]
            public DateTime Data { get; set; }

            [Required]
            [StringLength(150, MinimumLength = 1, ErrorMessage = "Màxim {1} caracters.")]
            [DataType(DataType.MultilineText)]
            public string Titol { get; set; }
            
            [AllowHtml]
            [DataType(DataType.MultilineText)]
            public string Descripcio { get; set; }

            [DataType(DataType.Url)]
            public string UrlNoticia { get; set; }

        }
    }

    [MetadataType(typeof(AvaluacioMeta))]
    public partial class Avaluacio
    {
        private class AvaluacioMeta
        {

            [Display(Name = "Nota primer Trimestre")]
            public Nullable<decimal> PrTrimestre { get; set; }


            [Display(Name = "Nota segon Trimestre")]
            public Nullable<decimal> SgTrimestre { get; set; }


            [Display(Name = "Nota tercer Trimestre")]
            public Nullable<decimal> TrTrimestre { get; set; }


            [Display(Name = "Nota final")]
            public Nullable<decimal> Final { get; set; }
        }
    }

    public enum EstatCita { 
        PendentProfessor,
        PendentPare,
        Acceptada,
        CanceladaPare,
        CanceladaProfessor,
        RebutjadaPare,
        RebutjadaProfessor

    }
    
    public partial class Cita {
        public void setEstat(EstatCita estat){
            this.Estat = (int)estat;            
        }
        public EstatCita getEstat() {
            return (EstatCita)this.Estat;
            
        }
        public string NomEstat {
            get {
                return Enum.GetName(typeof(EstatCita), (int)this.Estat);
            }
        }
    }

    
}