﻿using System;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Threading;
using System.Web.Mvc;
using WebMatrix.WebData;
using System.Web.Security;
using EscolaMVC.Models;

namespace EscolaMVC.Filters
{
    [AttributeUsage(AttributeTargets.Class | AttributeTargets.Method, AllowMultiple = false, Inherited = true)]
    public sealed class InitializeSimpleMembershipAttribute : ActionFilterAttribute
    {
        private static SimpleMembershipInitializer _initializer;
        private static object _initializerLock = new object();
        private static bool _isInitialized;

        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            // Ensure ASP.NET Simple Membership is initialized only once per app start
            LazyInitializer.EnsureInitialized(ref _initializer, ref _isInitialized, ref _initializerLock);
        }

        private class SimpleMembershipInitializer
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="SimpleMembershipInitializer" /> class.
            /// </summary>
            /// <exception cref="System.InvalidOperationException">The ASP.NET Simple Membership database could not be initialized. For more information, please see http://go.microsoft.com/fwlink/?LinkId=256588</exception>
            public SimpleMembershipInitializer()
            {
                Database.SetInitializer<UsersContext>(null);

                try
                {
                    using (var context = new UsersContext())
                    {
                        if (!context.Database.Exists())
                        {
                            // Create the SimpleMembership database without Entity Framework migration schema
                            ((IObjectContextAdapter)context).ObjectContext.CreateDatabase();
                        }
                    }

                    /*Default connection ha d'estar definit al web config com a la seccion connectionstrings*/
                    WebSecurity.InitializeDatabaseConnection("UsuarisEscola", "UserProfile", "UserId", "UserName", autoCreateTables: true);
                    
                    if (!WebSecurity.UserExists("Administrador"))
                        WebSecurity.CreateUserAndAccount("Administrador", "Administrador");

                    if (!Roles.RoleExists("Administradors"))
                        Roles.CreateRole("Administradors");

                    if (!Roles.IsUserInRole("Administrador", "Administradors"))
                        Roles.AddUserToRole("Administrador", "Administradors");

                    if (!Roles.RoleExists("Professors"))
                        Roles.CreateRole("Professors");

                    if (!Roles.IsUserInRole("Administrador", "Administradors"))
                        Roles.AddUserToRole("Administrador", "Administradors");
                    
                }
                catch (Exception ex)
                {
                    throw new InvalidOperationException("The ASP.NET Simple Membership database could not be initialized. For more information, please see http://go.microsoft.com/fwlink/?LinkId=256588", ex);
                }
            }
        }
    }
}
