using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Data.SqlClient;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;


namespace EscolaMVC.Models
{ 
    public class AdministratiuRepository : IAdministratiuRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";

        public string Error {
            get { return this._error; }
        }


        public IQueryable<Administratiu> All
        {
            get { return context.Administratiu; }
        }

        public IQueryable<Administratiu> AllIncluding(params Expression<Func<Administratiu, object>>[] includeProperties)
        {
            IQueryable<Administratiu> query = context.Administratiu;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Administratiu Find(int id)
        {
            return context.Administratiu.Find(id);
        }

        public void InsertOrUpdate(Administratiu administratiu)
        {
            if (administratiu.Id == default(int)) {
                // New entity
                context.Administratiu.Add(administratiu);
            } else {
                // Existing entity
                context.Entry(administratiu).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var administratiu = context.Administratiu.Find(id);
            context.Administratiu.Remove(administratiu);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                _error = ("Excepci no controlada al guardar les dades:" + ex.Message);
                while(ex.InnerException!=null){
                    ex = ex.InnerException;
                    if (ex is SqlException) {
                        _error = ("SqlServerError:" + ex.Message);
                        return false;
                    }
                }
                return false;
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IAdministratiuRepository : IDisposable
    {
        IQueryable<Administratiu> All { get; }
        IQueryable<Administratiu> AllIncluding(params Expression<Func<Administratiu, object>>[] includeProperties);
        Administratiu Find(int id);
        void InsertOrUpdate(Administratiu administratiu);
        void Delete(int id);
        bool Save();
        string Error{get;}
    }
}