using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class AlumneRepository : IAlumneRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";

        public string Error { get { return _error; } }

        public IQueryable<Alumne> All
        {
            get { return context.Alumne; }
        }

        public IQueryable<Alumne> AllIncluding(params Expression<Func<Alumne, object>>[] includeProperties)
        {
            IQueryable<Alumne> query = context.Alumne;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Alumne Find(int id)
        {
            return context.Alumne.Find(id);
        }

        public void InsertOrUpdate(Alumne alumne)
        {
            if (alumne.Id == default(int)) {
                // New entity
                context.Alumne.Add(alumne);
            } else {
                // Existing entity
                context.Entry(alumne).State = EntityState.Modified;
            }
        }
        public bool AfegirPare(Alumne alumne, int idpare) {
            Pare pare = context.Pare.Find(idpare);
            if (pare != null)
            {
                alumne.Pare.Add(pare);
                context.Entry(alumne).State = EntityState.Modified;
                return true;
            }
            else {
                _error = "No hi ha cap pare amb aquest Id.";
                return false;
            }
            
        }

        public bool EliminarPare(Alumne alumne, int idpare) {
            Pare pare = context.Pare.Find(idpare);
            if (pare == null) {
                this._error = "No existeix cap pare per aquest alumne amb el id indicat";
                return false;
            }
            pare.Alumne.Remove(alumne);
            return true;
        }

        public void Delete(int id)
        {
            var alumne = context.Alumne.Find(id);
            context.Alumne.Remove(alumne);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex) {
                this._error = ex.Message;
                return false;
            }
            
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IAlumneRepository : IDisposable
    {
        IQueryable<Alumne> All { get; }
        IQueryable<Alumne> AllIncluding(params Expression<Func<Alumne, object>>[] includeProperties);
        Alumne Find(int id);
        void InsertOrUpdate(Alumne alumne);
        bool EliminarPare(Alumne alumne, int idpare);
        bool AfegirPare(Alumne alumne,int idpare);
        void Delete(int id);
        bool Save();
        string Error { get; }
    }
}