using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class AssignaturaRepository : IAssignaturaRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";
        
        public string Error { get { return _error; } }

        public IQueryable<Assignatura> All
        {
            get { return context.Assignatura; }
        }

        public IQueryable<Assignatura> AllIncluding(params Expression<Func<Assignatura, object>>[] includeProperties)
        {
            IQueryable<Assignatura> query = context.Assignatura;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Assignatura Find(int id)
        {
            return context.Assignatura.Find(id);
        }

        public bool Insert(Assignatura assignatura)
        {
            bool trobat = context.Assignatura.Any(a => a.Nom == assignatura.Nom && a.Curs.Id == assignatura.Curs.Id);
            if (trobat) {
                _error = "Ja existeix una assignatura amb aquest nom per a aquest curs.";
                return false;
            }
            this.context.Entry(assignatura.Curs).State = EntityState.Unchanged;
            context.Assignatura.Add(assignatura);
            return true;
            
        }
        public bool Update(Assignatura assignatura, int idoldCurs, int idnouCurs)
        {
            bool trobat = context.Assignatura.Any(a => a.Nom == assignatura.Nom && a.Curs.Id == idnouCurs);
            if (trobat)
            {
                _error = "Ja existeix una assignatura amb aquest nom per a aquest curs.";
                return false;
            }
            context.Entry(assignatura).State = EntityState.Modified;
            var oldCurs = context.Curs.Find(idoldCurs);
            context.Curs.Attach(oldCurs);
            oldCurs.Assignatura.Remove(assignatura);
            var nouCurs = context.Curs.Find(idnouCurs);
            context.Curs.Attach(nouCurs);
            nouCurs.Assignatura.Add(assignatura);
            return true;
        }

        public void Delete(int id)
        {
            var assignatura = context.Assignatura.Find(id);
            context.Assignatura.Remove(assignatura);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                _error = ex.Message;
                return false;
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IAssignaturaRepository : IDisposable
    {
        IQueryable<Assignatura> All { get; }
        IQueryable<Assignatura> AllIncluding(params Expression<Func<Assignatura, object>>[] includeProperties);
        Assignatura Find(int id);
        bool Insert(Assignatura assignatura);
        bool Update(Assignatura assignatura, int oldCursId, int nouCursId);
        void Delete(int id);
        bool Save();
        string Error { get; }
    }
}