using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class AvaluacioRepository : IAvaluacioRepository
    {
        EscolaEntities context = new EscolaEntities();

        public IQueryable<Avaluacio> All
        {
            get { return context.Avaluacio; }
        }

        public IQueryable<Avaluacio> AllIncluding(params Expression<Func<Avaluacio, object>>[] includeProperties)
        {
            IQueryable<Avaluacio> query = context.Avaluacio;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Avaluacio Find(int id)
        {
            return context.Avaluacio.Find(id);
        }

        public void InsertOrUpdate(Avaluacio avaluacio)
        {
            if (avaluacio.Id == default(int)) {
                // New entity
                context.Avaluacio.Add(avaluacio);
            } else {
                // Existing entity
                context.Entry(avaluacio).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var avaluacio = context.Avaluacio.Find(id);
            context.Avaluacio.Remove(avaluacio);
        }

        public void CarregarNotes(IEnumerable<Avaluacio> notes) {
            foreach (Avaluacio avaluacio in notes) {
                Avaluacio dbAvaluacio = context.Avaluacio.Find(avaluacio.Id);
                if (avaluacio.PrTrimestre != null && avaluacio.PrTrimestre != -1)
                {
                    dbAvaluacio.PrTrimestre = avaluacio.PrTrimestre;
                }
                if (avaluacio.PrTrimestre == -1) {
                    dbAvaluacio.PrTrimestre = null;
                }


                if (avaluacio.SgTrimestre != null && avaluacio.SgTrimestre != -1)
                {
                    dbAvaluacio.SgTrimestre = avaluacio.SgTrimestre;
                }
                if (avaluacio.SgTrimestre == -1)
                {
                    dbAvaluacio.SgTrimestre = null;
                }


                if (avaluacio.TrTrimestre != null && avaluacio.TrTrimestre != -1)
                {
                    dbAvaluacio.TrTrimestre = avaluacio.TrTrimestre;
                }
                if (avaluacio.TrTrimestre == -1)
                {
                    dbAvaluacio.TrTrimestre = null;
                }


                if (avaluacio.Final != null && avaluacio.Final != -1)
                {
                    dbAvaluacio.Final = avaluacio.Final;
                }
                if (avaluacio.Final == -1)
                {
                    dbAvaluacio.Final = null;
                }
                context.Entry(dbAvaluacio).State = EntityState.Modified;
            }
        }
        

        public void Save()
        {
            context.SaveChanges();
        }

        public void Dispose() 
        {
            context.Dispose();
        }

        /*Genera avaluaci per a tots els alumnes matriculats en l'any proporcionat. Verifica que no hi hagin duplicats.*/
        public void Generar(int Any, out int generats)
        {
            generats = 0;
            var cursos = context.Curs.AsNoTracking().Where(curs=>curs.AnyCurs==Any);
            foreach(Curs curs in cursos){
                foreach (Classe classe in curs.Classe) {
                    foreach (Matricula matricula in classe.Matricula) {
                        context.Matricula.Attach(matricula);
                        foreach(Assignatura assignatura in curs.Assignatura){
                            context.Assignatura.Attach(assignatura);
                            Avaluacio avaluacio = new Avaluacio();
                            avaluacio.Assignatura = assignatura;
                            avaluacio.Matricula = matricula;
                            if(!context.Avaluacio.Any(aval=>aval.Assignatura.Id == assignatura.Id && aval.Matricula.Id == matricula.Id)){
                                InsertOrUpdate(avaluacio);
                                generats++;
                            }
                        }
                    }
                }    
            }
        }
    }

    public interface IAvaluacioRepository : IDisposable
    {
        IQueryable<Avaluacio> All { get; }
        IQueryable<Avaluacio> AllIncluding(params Expression<Func<Avaluacio, object>>[] includeProperties);
        Avaluacio Find(int id);
        void InsertOrUpdate(Avaluacio avaluacio);
        void CarregarNotes(IEnumerable<Avaluacio> notes);
        void Delete(int id);
        void Save();
        void Generar(int Any, out int generats);
    }
}