using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class ClasseRepository : IClasseRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = string.Empty;

        public string Error { get { return _error; } }

        public IQueryable<Classe> All
        {
            get { return context.Classe; }
        }

        public IQueryable<Classe> AllIncluding(params Expression<Func<Classe, object>>[] includeProperties)
        {
            IQueryable<Classe> query = context.Classe;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Classe Find(int id)
        {
            return context.Classe.Find(id);
        }

        public bool Insert(Classe classe)
        {
            bool trobat = context.Classe.Any(c => c.Lletra == classe.Lletra && c.Curs.Id == classe.Curs.Id);
            if (trobat) {
                _error = "No pot haver-hi registres duplicats.";
                return false;
            }
            this.context.Entry(classe.Curs).State = EntityState.Unchanged;
            context.Classe.Add(classe);
            return true;
        }

        public bool Update(Classe classe, int idoldCurs, int idnouCurs) {
            bool trobat = context.Classe.Any(c => c.Lletra == classe.Lletra && c.Curs.Id == idnouCurs);
            if (trobat) {
                _error = "No pot haver-hi registres duplicats.";
                return false;
            }
            context.Entry(classe).State = EntityState.Modified;
            var oldCurs = context.Curs.Find(idoldCurs);
            context.Curs.Attach(oldCurs);
            oldCurs.Classe.Remove(classe);
            var nouCurs = context.Curs.Find(idnouCurs);
            context.Curs.Attach(nouCurs);
            nouCurs.Classe.Add(classe);
            return true;
                
        }

        public void Delete(int id)
        {
            var classe = context.Classe.Find(id);
            context.Classe.Remove(classe);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch(Exception ex) {
                _error = ex.Message;
                return false;
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }


        public bool AssignarTutor(Classe classe, int ProfessorId)
        {       
                Professor tutor = context.Professor.Find(ProfessorId);
                if (tutor != null) {
                    classe.Professor = tutor;
                    context.Entry(classe).State = EntityState.Modified;
                    return true;
                }
            
            return false;
        }

        public bool EliminarTutor(Classe classe)
        {
            if (classe.Professor == null) {
                return true;
            }
            context.Entry(classe.Professor).State = EntityState.Unchanged;
            classe.Professor = null;
            context.Entry(classe).State = EntityState.Modified;
            return true;
        }
    }

    public interface IClasseRepository : IDisposable
    {
        IQueryable<Classe> All { get; }
        IQueryable<Classe> AllIncluding(params Expression<Func<Classe, object>>[] includeProperties);
        Classe Find(int id);
        bool Insert(Classe classe);
        bool Update(Classe classe, int oldCurs, int nouCurs);
        bool AssignarTutor(Classe classe, int ProfessorId);
        bool EliminarTutor(Classe classe);

        void Delete(int id);
        bool Save();
        string Error { get; }
    }
}