using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;
using System.Data.SqlClient;

namespace EscolaMVC.Models
{ 
    public class DocenciaRepository : IDocenciaRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";

        public string Error { get { return this._error; } }

        public IQueryable<Docencia> All
        {
            get { return context.Docencia; }
        }

        public IQueryable<Docencia> AllIncluding(params Expression<Func<Docencia, object>>[] includeProperties)
        {
            IQueryable<Docencia> query = context.Docencia;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Docencia Find(int id)
        {
            return context.Docencia.Find(id);
        }

        public void Generar(int any) {
            List<Curs> Cursos = context.Curs.AsNoTracking().Where(c => c.AnyCurs == any).ToList();
            foreach(Curs curs in Cursos){
                context.Entry(curs).State = EntityState.Unchanged;
                foreach (Classe classe in curs.Classe)
                {
                    context.Entry(classe).State = EntityState.Unchanged;
                    
                    foreach (Assignatura ass in curs.Assignatura) {
                        context.Entry(ass).State = EntityState.Unchanged;
                        Docencia docencia = new Docencia();
                        docencia.Assignatura = ass;
                        docencia.Classe = classe;
                        if(!context.Docencia.Any(doc=>doc.Assignatura.Id == docencia.Assignatura.Id && doc.Classe.Id == classe.Id)){
                            InsertOrUpdate(docencia);
                        }
                    }
                }
            }
            
            

        }

        public bool AfegirProfessor(int DocenciaId, int ProfessorId) {
            Professor professor = context.Professor.Find(ProfessorId);
            Docencia docencia = context.Docencia.Find(DocenciaId);
            if (professor != null && docencia != null)
            {
                context.Entry(professor).State = EntityState.Unchanged;
                docencia.Professor = professor;
                context.Entry(docencia).State = EntityState.Modified;
                return true;
            }
            else {
                _error = "No existeix la docncia o professor especificats.";
                return false;
            }
        }

        public void InsertOrUpdate(Docencia docencia)
        {
            if (docencia.Id == default(int)) {
                // New entity
                context.Docencia.Add(docencia);
            } else {
                // Existing entity
                context.Entry(docencia).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var docencia = context.Docencia.Find(id);
            context.Docencia.Remove(docencia);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex) {
                _error = ("Excepci no controlada al guardar les dades:" + ex.Message);
                while (ex.InnerException != null)
                {
                    ex = ex.InnerException;
                    if (ex is SqlException)
                    {
                        if(ex.Message.Contains("UQ_Docencia")){
                            _error = "Ja existeix una docncia per aquesta assignatura i classe.";
                        }
                        else _error = ("SqlServerError:" + ex.Message);
                        return false;
                    }
                }
                return false;     
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IDocenciaRepository : IDisposable
    {
        IQueryable<Docencia> All { get; }
        IQueryable<Docencia> AllIncluding(params Expression<Func<Docencia, object>>[] includeProperties);
        Docencia Find(int id);
        void Generar(int any);
        void InsertOrUpdate(Docencia docencia);
        bool AfegirProfessor(int DocenciaId, int ProfessorId);
        void Delete(int id);
        bool Save();
        string Error { get; }
    }
}