﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;
using System.Data.SqlClient;


namespace EscolaMVC.Models
{
    public interface IEscolaSingleTonRepository : IDisposable
    {
        Escola Escola { get; }
        bool Insert(Escola Escola);
        bool Update(Escola Escola);
        void Delete();
        bool Save();
        string Error { get; }
    }
    
    public class EscolaSingleTonRepository : IEscolaSingleTonRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";
        public string Error { get { return this._error; } }
        
        public Escola Escola
        {
            get {
                if(context.Escola.Any()){
                    return context.Escola.First();
                }
                else return null;
            }
        }

        public bool Insert(Escola Escola)
        {
            if(!context.Escola.Any()){
                context.Escola.Add(Escola);
                return true;
            }
            _error = "Ja existeix una escola";
            return false;
        }
        
        public bool Update(Escola Escola)
        {
            context.Entry(Escola).State = EntityState.Modified;
            return true;
        }

        public void Delete()
        {
            if (context.Escola.Any())
            {
                var escola = context.Escola.Find(0);
                context.Escola.Remove(escola);
            }
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                _error = ("Excepció no controlada al guardar les dades:" + ex.Message);
                while (ex.InnerException != null)
                {
                    ex = ex.InnerException;
                    if (ex is SqlException)
                    {
                        _error = ("SqlServerError:" + ex.Message);
                        if (ex.Message.Contains("PK_Escola"))
                        {
                            _error = "Només hi pot haver una escola.";
                        }
                        if (ex.Message.Contains("CK_Escola"))
                        {
                            _error = "Només hi pot haver una escola.";
                        }
                        return false;
                    }
                }
                return false;
            }
        }

        

        public void Dispose()
        {
            context.Dispose();
        }

        public static string NomEscola {
            get
            {
                using (EscolaEntities context = new EscolaEntities())
                {
                    string nomEscola = (context.Escola.Any() ? context.Escola.First().Nom : null);
                    return nomEscola;
                }
            }
        }
    }
}