using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using System.Data.SqlClient;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class MatriculaRepository : IMatriculaRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";

        public string Error {
            get { return _error; }
            set { _error = value; }
        }
        
        public IQueryable<Matricula> All
        {
            get { return context.Matricula; }
        }

        public IQueryable<Matricula> AllIncluding(params Expression<Func<Matricula, object>>[] includeProperties)
        {
            IQueryable<Matricula> query = context.Matricula;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Matricula Find(int id)
        {
            return context.Matricula.Find(id);
        }

        public void Insert(Matricula matricula, int ClasseId)
        {   
            Classe classe = context.Classe.Find(ClasseId);
            
            if (classe != null)
            {
                matricula.Classe = classe;
                context.Entry(classe).State = EntityState.Unchanged;
                if (matricula.Id == default(int))
                {
                    context.Matricula.Add(matricula);
                }
            }
        }
        public void CanviarMatricula(int MatriculaId, int Any , int ClasseId) {

                Matricula matricula = context.Matricula.Find(MatriculaId);
                matricula.Any = Any;
                Classe novaClasse = context.Classe.Find(ClasseId);

                if (novaClasse != null)
                {
                    matricula.Classe = novaClasse;
                    context.Entry(matricula.Classe).State = EntityState.Unchanged;
                    context.Entry(matricula).State = EntityState.Modified;
                }
                else {
                    if (ClasseId == -1) {
                        context.Entry(matricula.Classe).State = EntityState.Unchanged;
                        matricula.Classe = null;
                        context.Entry(matricula).State = EntityState.Modified;
                    }
                }

            
            
        }

        public void Delete(int id)
        {
            var matricula = context.Matricula.Find(id);
            context.Matricula.Remove(matricula);
        }

        /*Genera matrcula sense classe per a tots el alumnes i any que no tenen matrcula encara*/
        public void Generar(int any){
            foreach (Alumne alumne in context.Alumne) {
                bool matriculat = context.Matricula.Any(m => m.Alumne.Id == alumne.Id && m.Any == any && m.Classe.Id != null);
                if (!matriculat) {
                    Matricula matricula = new Matricula();
                    matricula.Any = any;
                    matricula.Alumne = alumne;
                    context.Entry(alumne).State = EntityState.Unchanged;
                    context.Matricula.Add(matricula);
                }
            }
        }

        public bool AssignarClasse(int MatriculaId, int ClasseId){
            Matricula matricula = context.Matricula.Find(MatriculaId);
            Classe classe = context.Classe.Find(ClasseId);
            if (matricula != null && classe != null)
            {
                context.Entry(classe).State = EntityState.Unchanged;
                matricula.Classe = classe;
                context.Entry(matricula).State = EntityState.Modified;
                return true;
            }
            else {
                _error = "No existeix cap matrcula amb aquest ID o Classe. ";
                return false;
            }
            
            


        }

        public bool Save()
        {

            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                _error = ("Excepci no controlada al guardar les dades:" + ex.Message);
                while (ex.InnerException != null)
                {
                    ex = ex.InnerException;
                    if (ex is SqlException)
                    {
                        if (ex.Message.Contains("IX_Matricula"))
                        {
                            _error = "Ja existeix una matricula per amb aquest alumne i any.";
                        }
                        else _error = ("SqlServerError:" + ex.Message);
                        return false;
                    }
                }
                return false;
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IMatriculaRepository : IDisposable
    {
        IQueryable<Matricula> All { get; }
        IQueryable<Matricula> AllIncluding(params Expression<Func<Matricula, object>>[] includeProperties);
        Matricula Find(int id);
        void Insert(Matricula matricula, int ClasseId);
        void CanviarMatricula(int MatriculaId,int Any, int ClasseId);
        void Generar(int any);
        bool AssignarClasse(int MatriculaId, int ClasseId);
        void Delete(int id);
        bool Save();
        string Error { get; set; }
    }
}