using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class NoticiaRepository : INoticiaRepository
    {
        EscolaEntities context = new EscolaEntities();

        public IQueryable<Noticia> All
        {
            get { return context.Noticia; }
        }

        public IQueryable<Noticia> AllIncluding(params Expression<Func<Noticia, object>>[] includeProperties)
        {
            IQueryable<Noticia> query = context.Noticia;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Noticia Find(int id)
        {
            return context.Noticia.Find(id);
        }

        public void InsertOrUpdate(Noticia noticia)
        {
            if (noticia.ID == default(int)) {
                // New entity
                context.Noticia.Add(noticia);
            } else {
                // Existing entity
                context.Entry(noticia).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var noticia = context.Noticia.Find(id);
            context.Noticia.Remove(noticia);
        }

        public void Save()
        {
            context.SaveChanges();
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface INoticiaRepository : IDisposable
    {
        IQueryable<Noticia> All { get; }
        IQueryable<Noticia> AllIncluding(params Expression<Func<Noticia, object>>[] includeProperties);
        Noticia Find(int id);
        void InsertOrUpdate(Noticia noticia);
        void Delete(int id);
        void Save();
    }
}