using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Data.SqlClient;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class PareRepository : IPareRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";

        public string Error
        {
            get { return this._error; }
        }

        public IQueryable<Pare> All
        {
            get { return context.Pare; }
        }

        public IQueryable<Pare> AllIncluding(params Expression<Func<Pare, object>>[] includeProperties)
        {
            IQueryable<Pare> query = context.Pare;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Pare Find(int id)
        {
            return context.Pare.Find(id);
        }

        public void InsertOrUpdate(Pare pare)
        {
            if (pare.Id == default(int)) {
                // New entity
                context.Pare.Add(pare);
            } else {
                // Existing entity
                context.Entry(pare).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var pare = context.Pare.Find(id);
            context.Pare.Remove(pare);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch (Exception ex)
            {
                _error = ("Excepci no controlada al guardar les dades:" + ex.Message);
                while (ex.InnerException != null)
                {
                    ex = ex.InnerException;
                    if (ex is SqlException)
                    {
                        _error = ("SqlServerError:" + ex.Message);
                        return false;
                    }
                }
                return false;
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IPareRepository : IDisposable
    {
        IQueryable<Pare> All { get; }
        IQueryable<Pare> AllIncluding(params Expression<Func<Pare, object>>[] includeProperties);
        Pare Find(int id);
        void InsertOrUpdate(Pare pare);
        void Delete(int id);
        bool Save();
        string Error { get; }
    }
}