using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Data.SqlClient;
using System.Linq;
using System.Linq.Expressions;
using System.Web;
using EscolaMVC.Entitats;

namespace EscolaMVC.Models
{ 
    public class ProfessorRepository : IProfessorRepository
    {
        EscolaEntities context = new EscolaEntities();
        string _error = "";
        public string Error
        {
            get { return this._error; }
        }

        public IQueryable<Professor> All
        {
            get { return context.Professor; }
        }

        public IQueryable<Professor> AllIncluding(params Expression<Func<Professor, object>>[] includeProperties)
        {
            IQueryable<Professor> query = context.Professor;
            foreach (var includeProperty in includeProperties) {
                query = query.Include(includeProperty);
            }
            return query;
        }

        public Professor Find(int id)
        {
            return context.Professor.Find(id);
        }

        public void InsertOrUpdate(Professor professor)
        {
            if (professor.Id == default(int)) {
                // New entity
                context.Professor.Add(professor);
            } else {
                // Existing entity
                context.Entry(professor).State = EntityState.Modified;
            }
        }

        public void Delete(int id)
        {
            var professor = context.Professor.Find(id);
            context.Professor.Remove(professor);
        }

        public bool Save()
        {
            try
            {
                context.SaveChanges();
                return true;
            }
            catch(Exception ex) {
                _error = ("Excepci no controlada al guardar les dades:" + ex.Message);
                while (ex.InnerException != null)
                {
                    ex = ex.InnerException;
                    if (ex is SqlException)
                    {
                        _error = ("SqlServerError:" + ex.Message);
                        return false;
                    }
                }
                return false;
            
            }
        }

        public void Dispose() 
        {
            context.Dispose();
        }
    }

    public interface IProfessorRepository : IDisposable
    {
        IQueryable<Professor> All { get; }
        IQueryable<Professor> AllIncluding(params Expression<Func<Professor, object>>[] includeProperties);
        Professor Find(int id);
        void InsertOrUpdate(Professor professor);
        void Delete(int id);
        bool Save();
        string Error { get; }
    }
}