﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Transactions;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using WebMatrix.WebData;
using EscolaMVC.Filters;
using EscolaMVC.Models;
using EscolaMVC.Entitats;
using System.Data;
using System.Data.Entity;
using System.Net.Mail;

namespace EscolaMVC.Services
{

    /* Serveis de domini*/
    #region domain services

    public class GestioUsuaris
    {
        public bool ResetPassword(string usuari, string password, out string error)
        {
            error = string.Empty;
            if (WebSecurity.Initialized)
            {
                if (WebSecurity.UserExists(usuari))
                {
                    string token = WebSecurity.GeneratePasswordResetToken(usuari);
                    return WebSecurity.ResetPassword(token, password);
                }
                else
                {
                    error = "Usuari no trobat";
                    return false;
                }
            }
            else {
                error = "Gestor de seguretat no inicialitzat.";
                return false; 
            }

        }

        public int CrearUsuariAp(string usuari, string password, string rol, out string error) {
            error = string.Empty;
            int idusuari = -1;
            if (WebSecurity.Initialized)
            {
                if (!WebSecurity.UserExists(usuari))
                {
                    if (!Roles.RoleExists(rol))
                    {
                        error = "El rol no existeix";
                        return -1;
                    }
                    WebSecurity.CreateUserAndAccount(usuari, password);
                    Roles.AddUserToRole(usuari, rol);
                    return WebSecurity.GetUserId(usuari);
                }
                else
                {
                    error = "L'usuari ja existeix";
                }
            }
            else
            {
                error = "Gestor de seguretat no inicialitzat";
            }
            return idusuari;

        }
        
        public bool CrearUsuari(Administratiu admin, out string error) {
            string usuari = admin.Correu;
            string password = admin.Correu;
            string rol = EscolaMVC.Entitats.RolsUsuari.RolAdministratiu;
            error = string.Empty;
            using (EscolaEntities escola = new Entitats.EscolaEntities())
            {
                bool trobat = escola.Administratiu.Where(admins => admins.DNI == admin.DNI).Count() == 1;
                if (trobat)
                {
                    error = "L'usuari ja existeix";
                    return false;
                }
                int idusuari = CrearUsuariAp(usuari, password, rol, out error);
                if (idusuari == -1)
                {
                    return false;
                }
                UsuariAp up = escola.UserProfile.Find(idusuari);
                admin.UserProfile = up;
                escola.Administratiu.Add(admin);
                escola.Entry(up).State = EntityState.Unchanged;
                try
                {
                    escola.SaveChanges();
                    return true;
                }
                catch(Exception ex)
                {
                    error = ex.Message;
                    string errorEsborra = string.Empty;
                    EsborrarUsuari(usuari, out errorEsborra);
                    error += ". " + errorEsborra;
                }
                return false;
            }
        }

        public bool CrearUsuari(Professor professor, out string error)
        {
            string usuari = professor.Correu;
            string password = professor.Correu;
            string rol = EscolaMVC.Entitats.RolsUsuari.RolProfessor;
            error = string.Empty;
            using (EscolaEntities escola = new Entitats.EscolaEntities())
            {
                bool trobat = escola.Professor.Where(professors => professors.DNI == professor.DNI).Count() == 1;
                if (trobat)
                {
                    error = "L'usuari ja existeix";
                    return false;
                }
                int idusuari = CrearUsuariAp(usuari, password, rol, out error);
                if (idusuari == -1)
                {
                    return false;
                }
                UsuariAp up = escola.UserProfile.Find(idusuari);
                professor.UserProfile = up;
                escola.Professor.Add(professor);
                escola.Entry(up).State = EntityState.Unchanged;
                escola.SaveChanges();
                return true;
            }

        }

        public bool CrearUsuari(Pare pare, out string error)
        {
            string usuari = pare.Correu;
            string password = pare.Correu;
            string rol = EscolaMVC.Entitats.RolsUsuari.RolPare;
            error = string.Empty;
            using (EscolaEntities escola = new Entitats.EscolaEntities())
            {
                bool trobat = escola.Administratiu.Where(pares => pares.DNI == pare.DNI).Count() == 1;
                if (trobat)
                {
                    error = "L'usuari ja existeix";
                    return false;
                }
                int idusuari = CrearUsuariAp(usuari, password, rol, out error);
                if (idusuari == -1)
                {
                    return false;
                }
                UsuariAp up = escola.UserProfile.Find(idusuari);
                pare.UserProfile = up;
                escola.Pare.Add(pare);
                escola.Entry(up).State = EntityState.Unchanged;
                escola.SaveChanges();
                return true;
            }

        }
        
        public bool EsborrarUsuari(string UserName, out string error)
        {
            error = string.Empty;
            if (WebSecurity.Initialized)
            {
                if (!WebSecurity.UserExists(UserName))
                {
                    error = "L'usuari no existeix";
                    return false;
                }
                else
                {
                    try
                    {
                        string[] rolsusuari = Roles.GetRolesForUser(UserName);
                        if (rolsusuari.Count() > 0)
                        {
                            Roles.RemoveUserFromRoles(UserName, rolsusuari);
                        }
                        ((SimpleMembershipProvider)Membership.Provider).DeleteAccount(UserName);
                        return Membership.DeleteUser(UserName, true);
                    }
                    catch (Exception ex)
                    {
                        error = ex.Message;
                        return false;
                    }
                }
            }

            error = "Gestor de seguretat no inicialitzat.";
            return false;

        }
    }

    public class PortalPares : IDisposable{

        EscolaEntities context;

        public PortalPares() {
            context = new EscolaEntities();
        }
        
        public Pare getPareUser(string UserName)
        {   
            try
            {
                UsuariAp usuariValidat = context.UserProfile.Where(usuari => usuari.UserName == UserName).First();
                return usuariValidat.Pare.First();
            }
            catch
            {
                return new Pare();
            }
        }

        public IEnumerable<FillViewModel> getFillsAmbAvaluacio(Pare ElPare, int? pAny)
        {
            if (!pAny.HasValue) { 
                pAny = DateTime.Today.Year;
                
                /* Descomentar después de la demo.  */
                //if (DateTime.Today.Month >= 1 && DateTime.Today.Month < 9) {
                //    pAny = DateTime.Today.Year -1;
                //}
            }
            List<FillViewModel> fillsVm = new List<FillViewModel>();
            var fills = ElPare.Alumne.AsEnumerable();
            foreach (var fill in fills)
            {
                FillViewModel FillVm = new FillViewModel();
                FillVm.Id = fill.Id;
                FillVm.NomiCognoms = fill.Nom + " " + fill.Cognoms;
                Matricula matricula = fill.Matricula.Where(m => m.Any == pAny).FirstOrDefault();
                /* Alumne no matriculat no té classe */
                FillVm.Classe = matricula!=null?matricula.Classe : new Classe();
                List<AvaluacioViewModel> avaluacions = new List<AvaluacioViewModel>();
                var avals = context.Avaluacio.Where(aval => aval.Matricula.Alumne.Id == fill.Id && aval.Matricula.Any == pAny);
                foreach (var aval in avals)
                {
                    AvaluacioViewModel avm = new AvaluacioViewModel();
                    avm.AvaluacioId = aval.Id;
                    avm.IdAssig = aval.Assignatura.Id;
                    avm.NomAssignatura = aval.Assignatura.Nom;
                    avm.Comentaris = aval.ComentariAvaluacio.AsEnumerable();
                    if (aval.Assignatura.Docencia.Any(doc => doc.Classe.Id == FillVm.Classe.Id && doc.Professor != null))
                    {
                        var docencia = aval.Assignatura.Docencia.Where(doc => doc.Classe.Id == FillVm.Classe.Id).First();
                        avm.IdProfe = docencia.Professor.Id;
                        avm.Profe = docencia.Professor.Nom + " " + docencia.Professor.Cognoms;
                    }
                    avm.NotaPrimerTrimestre = aval.PrTrimestre;
                    avm.NotaSegonTrimestre = aval.SgTrimestre;
                    avm.NotaTercerTrimestre = aval.TrTrimestre;
                    avm.NotaFinalTrimestre = aval.Final;
                    avaluacions.Add(avm);
                }
                FillVm.Avaluacions = avaluacions;
                fillsVm.Add(FillVm);
            }
            return fillsVm;
        }

        public PortalParesViewModel getPortalPares(string UserName) {
            Pare PareUsuari = this.getPareUser(UserName);
            PortalParesViewModel portalpares = new PortalParesViewModel();
            portalpares.Pare = PareUsuari;
            portalpares.Fills = this.getFillsAmbAvaluacio(PareUsuari, null);
            return portalpares;
        }


        public void Dispose()
        {
            if (context != null) {
                context.Dispose();
            }
        }

        public IEnumerable<Cita> getCitasPare(int PareId, int year, int month)
        {
            var fills = context.Pare.Find(PareId).Alumne;
            var idfills = fills.Select(fill=>fill.Id).AsEnumerable();
            DateTime inici = new DateTime(year, month, 1);
            int diesmes = DateTime.DaysInMonth(year, month);
            DateTime fi = new DateTime(year, month, diesmes);
            var citas = context.Cita.Where(cita=> 
                 idfills.Contains(cita.Alumne.Id) && 
                ((cita.Dia.HasValue? (cita.Dia.Value >= inici && cita.Dia.Value <= fi) : true))
            ).AsEnumerable();
            return citas;
                
        }

       

        public bool eliminarCita(int id, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null) {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (!(cita.getEstat() == EstatCita.PendentProfessor || cita.getEstat() == EstatCita.CanceladaProfessor || cita.getEstat()== EstatCita.RebutjadaProfessor)) {
                error = "Només es poden eliminar les cites pendents o rebutjades";
                return false;
            }
            context.Cita.Remove(cita);
            context.SaveChanges();
            return true;
        }

        public bool cancelarCita(int id, string motius, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (cita.getEstat() != EstatCita.Acceptada)
            {
                error = "Només es poden cancel·lar les cites acceptades";
                return false;
            }
            cita.Historic = cita.Historic + "\n Motiu de cancel·lació: " + motius;
            cita.setEstat(EstatCita.CanceladaPare);
            context.SaveChanges();
            return true;
        }

        public bool rebutjarCita(int id, string motius, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (cita.getEstat() != EstatCita.PendentPare)
            {
                error = "Només es poden rebutjar les cites pendents de la seva aprovació";
                return false;
            }
            cita.setEstat(EstatCita.RebutjadaPare);
            cita.Historic = cita.Historic + "\n Motiu de cancel·lació: " + motius;
            context.SaveChanges();
            return true;
        }

        public bool acceptarCita(int id, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (cita.getEstat() != EstatCita.PendentPare)
            {
                error = "Només es poden acceptar les cites pendents de la seva aprovació";
                return false;
            }
            cita.setEstat(EstatCita.Acceptada);
            context.SaveChanges();
            return true;
        }

        public string getNomProfessorCita(int idprofe)
        {
            Professor profe = context.Professor.Find(idprofe);
            return profe.Cognoms + ", " + profe.Nom;
            
        }

        public string getNomFill(int idalumne)
        {
            Alumne alu = context.Alumne.Find(idalumne);
            return alu.Cognoms + ", " + alu.Nom;
        }

        public void citarProfessor(int idprofessor, int idalumne, DateTime dia,string comentari, string UserName)
        {
            Pare pare = getPareUser(UserName);
            Cita cita = new Cita();
            cita.setEstat(EstatCita.PendentProfessor);
            cita.Dia = dia;
            cita.Comentari = comentari;
            cita.Historic = string.Format("{0}: {1}, {2} amb e-mail:{3} demana la cita.", DateTime.Now.ToShortDateString(), pare.Cognoms, pare.Nom, pare.Correu);
            Professor profe = context.Professor.Find(idprofessor);
            Alumne alumne = context.Alumne.Find(idalumne);
            cita.Professor = profe;
            cita.Alumne = alumne;
            context.Entry(profe).State = EntityState.Unchanged;
            context.Entry(alumne).State = EntityState.Unchanged;
            context.Cita.Add(cita);
            context.SaveChanges();
            
        }

        
    }

    public class PortalProfessors : IDisposable {
        EscolaEntities context;

        public PortalProfessors() {
            context = new EscolaEntities();
        }


        public Professor getProfessorUser(string UserName) {
            try
            {
                UsuariAp usuariValidat = context.UserProfile.Where(usuari => usuari.UserName == UserName).First();
                return usuariValidat.Professor.First();
            }
            catch
            {
                return new Professor();
            }
        }

        public void Dispose()
        {
            if (context != null)
            {
                context.Dispose();
            }
        }

        public PortalProfessorViewModel getPortal(string UserName)
        {
            PortalProfessorViewModel portalVm = new PortalProfessorViewModel();
            Professor profe = getProfessorUser(UserName);
            portalVm.Professor = profe.Nom + " " + profe.Cognoms;
            portalVm.ProfessorId = profe.Id;
            var docencia = profe.Docencia;
            portalVm.Docencia = docencia.AsEnumerable();
            return portalVm;
        }

        public bool guardarAvaluacio(int MatriculaId, int AssignaturaId, string t1, string t2, string t3, string t4, out int idaval)
        {
            idaval = -1;
            try
            {
                decimal? dt1 = null;
                decimal? dt2 = null;
                decimal? dt3 = null;
                decimal? dt4 = null;
                if (t1 != null) dt1 = decimal.Parse(t1);
                if (t2 != null) dt2 = decimal.Parse(t2);
                if (t3 != null) dt3 = decimal.Parse(t3);
                if (t4 != null) dt4 = decimal.Parse(t4);
                
                
                Avaluacio avaluacio = context.Avaluacio.Where(aval => aval.Assignatura.Id == AssignaturaId && aval.Matricula.Id == MatriculaId).FirstOrDefault();
                if (avaluacio != null)
                {
                    avaluacio.PrTrimestre = dt1;
                    avaluacio.SgTrimestre = dt2;
                    avaluacio.TrTrimestre = dt3;
                    avaluacio.Final = dt4;
                    context.Entry(avaluacio).State = EntityState.Modified;
                    
                }
                else {
                    avaluacio = new Avaluacio();
                    avaluacio.PrTrimestre = dt1;
                    avaluacio.SgTrimestre = dt2;
                    avaluacio.TrTrimestre = dt3;
                    avaluacio.Final = dt4;
                    Assignatura assig = context.Assignatura.Find(AssignaturaId);
                    context.Entry(assig).State = EntityState.Unchanged;
                    Matricula matricula = context.Matricula.Find(MatriculaId);
                    context.Entry(matricula).State = EntityState.Unchanged;
                    avaluacio.Assignatura = assig;
                    avaluacio.Matricula = matricula;
                    context.Avaluacio.Add(avaluacio);
                }
                context.SaveChanges();
                idaval = avaluacio.Id;
                return true;
            }
            catch {
                return false;
            }
            
        }

        public IEnumerable<ComentariAvaluacio> getComentarisAvaluacio(int AvaluacioId)
        {
            return context.ComentariAvaluacio.Where(comentari => comentari.Avaluacio.Id == AvaluacioId);
        }

        public void eliminarComentari(int ComentariId)
        {
            ComentariAvaluacio comentari = context.ComentariAvaluacio.Find(ComentariId);
            if (comentari != null)
            {
                context.ComentariAvaluacio.Remove(comentari);
                context.SaveChanges();
            }
        }

        public ComentariAvaluacio altaComentari(int AvaluacioId, string comentari)
        {
            try
            {
                Avaluacio aval = context.Avaluacio.Find(AvaluacioId);
                context.Entry(aval).State = EntityState.Unchanged;
                ComentariAvaluacio noucomentari = new ComentariAvaluacio();
                noucomentari.Data = DateTime.Today;
                noucomentari.Avaluacio = aval;
                noucomentari.Comentari = comentari;
                noucomentari.Tipus = "comentari";
                context.ComentariAvaluacio.Add(noucomentari);
                context.SaveChanges();
                return noucomentari;
            }
            catch { 
                return new ComentariAvaluacio();
            }
        }

        public IEnumerable<Cita> getCitasProfessor(int ProfessorId, int year, int month)
        {
            DateTime inici = new DateTime(year, month, 1);
            int diesmes = DateTime.DaysInMonth(year, month);
            DateTime fi = new DateTime(year, month, diesmes);
            var citas = context.Cita.Where(cita =>
                 cita.Professor.Id == ProfessorId &&
                ((cita.Dia.HasValue ? (cita.Dia.Value >= inici && cita.Dia.Value <= fi) : true))
            ).AsEnumerable();
            return citas;
        }

        public bool eliminarCita(int id, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (!(cita.getEstat() == EstatCita.PendentPare || cita.getEstat() == EstatCita.CanceladaPare || cita.getEstat() == EstatCita.RebutjadaPare))
            {
                error = "Només es poden eliminar les cites pendents o rebutjades";
                return false;
            }
            context.Cita.Remove(cita);
            context.SaveChanges();
            return true;
        }

        public bool cancelarCita(int id, string motius, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (cita.getEstat() != EstatCita.Acceptada)
            {
                error = "Només es poden cancel·lar les cites acceptades";
                return false;
            }
            cita.Historic = cita.Historic + "\n Motiu de cancel·lació: " + motius;
            cita.setEstat(EstatCita.CanceladaProfessor);
            context.SaveChanges();
            return true;
        }

        public bool rebutjarCita(int id, string motius, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (cita.getEstat() != EstatCita.PendentPare)
            {
                error = "Només es poden rebutjar les cites pendents de la seva aprovació";
                return false;
            }
            cita.setEstat(EstatCita.RebutjadaProfessor);
            cita.Historic = cita.Historic + "\n Motiu de cancel·lació: " + motius;
            context.SaveChanges();
            return true;
        }

        public bool acceptarCita(int id, out string error)
        {
            error = "";
            Cita cita = context.Cita.Find(id);
            if (cita == null)
            {
                error = "No existeix cap cita amb aquest id";
                return false;
            }
            if (cita.getEstat() != EstatCita.PendentProfessor)
            {
                error = "Només es poden acceptar les cites pendents de la seva aprovació";
                return false;
            }
            cita.setEstat(EstatCita.Acceptada);
            context.SaveChanges();
            return true;
        }

        public string getNomAlumne(int idalumne)
        {
            Alumne alu = context.Alumne.Find(idalumne);
            return alu.Cognoms + ", " + alu.Nom;
        }

        public string[] getNomPares(int idalumne)
        {
            return context.Alumne.Find(idalumne).Pare.Select(p=> p.Cognoms+ ", " + p.Nom).ToArray();
        }

        public void citarParesAlumne(int idprofessor, int idalumne, DateTime dia, string comentari, string UserName)
        {
            Professor profe = context.Professor.Find(idprofessor);
            Cita cita = new Cita();
            cita.setEstat(EstatCita.PendentPare);
            cita.Dia = dia;
            cita.Comentari = comentari;
            cita.Historic = string.Format("{0}: {1}, {2} amb e-mail:{3} demana la cita.", DateTime.Now.ToShortDateString(), profe.Cognoms, profe.Nom, profe.Correu);
            Alumne alumne = context.Alumne.Find(idalumne);
            cita.Professor = profe;
            cita.Alumne = alumne;
            context.Entry(profe).State = EntityState.Unchanged;
            context.Entry(alumne).State = EntityState.Unchanged;
            context.Cita.Add(cita);
            context.SaveChanges();
        }
    }

    #endregion



    /* Serveis externs*/
    #region external services
    public interface ISmtpClientConfig {
        string Servidor { get;}
        string UserName { get;}
        string Correu { get; }
        string Password { get;}
        int Port { get; }
        bool SSL { get; }
    }

    
    public class WebConfigSmtpClient : ISmtpClientConfig
    {
        /* Exemple de configuració al webConfig */
        //<smtpClient servidor="smtp.gmail.com" usuari="paucasalstfc@gmail.com" password="gestioEscolar" correu="paucasalstfc@gmail.com" port="587" ssl="Sí"/>
        SmtpClientSection configuracio;

        /// <summary>
        /// Initializes a new instance of the <see cref="WebConfigSmtpClient" /> class.
        /// </summary>
        /// <exception cref="System.Exception">Error en la configuració dels paràmetres del servidor de correu, posis en contacte amb l'administrador del lloc.</exception>
        public WebConfigSmtpClient() {
            try
            {
                this.configuracio = System.Configuration.ConfigurationManager.GetSection("smtpClient") as SmtpClientSection;
            }
            catch {
                throw new Exception("Error en la configuració dels paràmetres del servidor de correu, posis en contacte amb l'administrador del lloc.");
            }
        }
        
        public string Servidor
        {
            get { return configuracio.Servidor; }
        }

        public string UserName
        {
            get { return configuracio.Usuari; }
        }

        public string Correu
        {
            get { return configuracio.Correu; }
        }

        public string Password
        {
            get { return configuracio.Password; }
        }

        public int Port
        {
            get { return configuracio.Port; }
        }

        public bool SSL
        {
            get {
                return configuracio.SSL;
            }
        }
    }


    /// <summary>
    /// Aquesta classe serveix per enviar correus, 
    /// la creadora rep una interfície amb la configuració del client smtp. 
    /// </summary>
    public class Correu {
        string Servidor; 
        string UserName;
        string CorreuBustia;
        string Password;
        int Port;
        bool SSL;

        /// <summary>
        /// Initializes a new instance of the <see cref="Correu" /> class.
        /// </summary>
        /// <param name="smtpClientConfig">The SMTP client config.</param>
        public Correu(ISmtpClientConfig smtpClientConfig) {
            Servidor = smtpClientConfig.Servidor;
            UserName = smtpClientConfig.UserName;
            CorreuBustia = smtpClientConfig.Correu;
            Password = smtpClientConfig.Password;
            Port = smtpClientConfig.Port;
            SSL = smtpClientConfig.SSL;
        }
        
        
        /// <summary>
        /// Envia un correu a l'adreça a la bústia amb els paràmetres indicats. 
        /// </summary>
        /// <param name="from">Indica quí fa l'enviament, es copia a replyTo.</param>
        /// <param name="nomicognoms">Nom i cognoms de l'usuari que envia el formulari.</param>
        /// <param name="assumpte">Assumpte del missatge</param>
        /// <param name="cos">Cos del missatge</param>
        /// <param name="error">Si no es pot validar contra el servidor retornem false i l'error</param>
        /// <returns>bool</returns>
        public bool EnviarCorreu(string from, string nomicognoms, string assumpte, string cos, out string error) {
            
            error = string.Empty;

            SmtpClient client = null;
            try
            {
                client = new SmtpClient(Servidor, Port);
                client.UseDefaultCredentials = false;
                client.Credentials = new System.Net.NetworkCredential(UserName, Password);
                client.EnableSsl = SSL;
                client.DeliveryMethod = SmtpDeliveryMethod.Network;

                MailMessage message = new MailMessage(from, CorreuBustia);
                message.Subject = string.Format(@"Portal Escola: Usuari {0}. {1}",nomicognoms,assumpte);
                message.Body = cos;
                message.ReplyToList.Add(from);
                client.Send(message);
                return true;
            }

            // This catches the exceptions, if any.
            catch (Exception ex)
            {
                error = ("No s'ha pogut realitzar l'enviament: " + ex.Message);
            }
            finally {
                if (client != null) {
                    client.Dispose();
                }
            }
            return false;
        }
    }

    #endregion
}